#backup betaTable 
betaTableOriginal <- betaTable

# only chr 1 - 21
#betaTable <- betaTable[which(betaTable$chr %in% seq(1,22,1)),]
betaTable <- betaTable[which(betaTable$chr %in% seq(1,22,1 ) | 
                               betaTable$chr %in% paste0("chr",seq(1,22,1 ))  ),]


## add ploidy information
cat(paste("[",Sys.time() ,"] Apply ploidy correction\n",sep=""))
betaTable <- merge(x = betaTable, y = ploidyTable, by.x = "sample", by.y = "sample")
betaTable$log2.plCorr <- betaTable$log2 - betaTable$log2shift

## add error information
cat(paste("[",Sys.time() ,"] Compute errors around beta estimates\n",sep=""))
betaTable.ext.list <- by(betaTable, INDICES = betaTable$sample, FUN = addErrorToBetaTable, errorTable = errorTable, ncores = NsamplesToProcessInParallel * perSampleCores )
betaTable.ext <- fromListToDF(betaTable.ext.list)

## compute adm global using cnA and cnB
## now works only with parental disomy cases
#id<-32
computeAdmGlobal2D <- function(id, samples, betaTable.ext, min_nsnps, min_cov, perSampleCores){
  #cat(id,"\n")
  # default output
  adm.global.this <- data.frame(row.names = samples[id])
  adm.global.this$sample <- samples[id]
  adm.global.this$adm <- NA
  adm.global.this$adm.min <- NA
  adm.global.this$adm.max <- NA
  adm.global.this$n.segs <- NA
  adm.global.this$n.SNPs <- NA
  
  
  betaTable.this <- betaTable.ext[which(betaTable.ext$sample == samples[id]),]
  betaTable.this <- betaTable.this[which(betaTable.this$nsnp >= min_nsnps & betaTable.this$cov >= min_cov),]
  
  if (is.na(betaTable.this$ploidy[1])){
    return(adm.global.this)
  }
  
  ## compute cnA and cnB wiht adm.global = 0 
  betaTable.this.ext <- extendBetaTableWithCopyNumbers(BetaTable = betaTable.this, G =0, ncores = perSampleCores)
  #rownames(betaTable.ext)
  
  ## to improve and use to compile reports (works only with sequential?)
#   plot(betaTable.this.ext$cnA,betaTable.this.ext$cnB,pch=20,xlim=c(0,2),ylim=c(0,2))
#   points(betaTable.this.ext$cnA.betaMax,betaTable.this.ext$cnB.betaMax,pch=20,col="orange4")
#   points(betaTable.this.ext$cnA.betaMin,betaTable.this.ext$cnB.betaMin,pch=20,col="blue4")
#   abline(v=seq(1,6,1),lty=2,col="gray60")
#   abline(h=seq(1,6,1),lty=2,col="gray60")
#   
  ## find maximum cnB value for the low cluster
  localBeta <- betaTable.this.ext[which(betaTable.this.ext$cnB < 1),]
  if (nrow(localBeta) < 2){
    return(adm.global.this)
  }
  
  localBeta <- localBeta[with(localBeta, expr = order(cnB)),]
  localBeta.next <- rbind(localBeta[seq(2,nrow(localBeta),1),],rep(NA,ncol(localBeta)))
  colnames(localBeta.next) <- paste0("next_",colnames(localBeta.next))
  localBeta <- cbind(localBeta,localBeta.next)
  localBeta$max_cnB_current <- pmax(localBeta$cnB,localBeta$cnB.betaMin,localBeta$cnB.betaMax, na.rm = T)
  localBeta$min_cnB_next <- pmin(localBeta$next_cnB,localBeta$next_cnB.betaMin,localBeta$next_cnB.betaMax,na.rm = T)
  max.cnB <- localBeta$cnB[min(which(localBeta$max_cnB_current < localBeta$min_cnB_next ))]
  
  localBeta.good <- localBeta[which(localBeta$cnB <= max.cnB ),]
  
  if (nrow(localBeta.good) > 0){
    G.seq <- seq(0,0.99,0.01)
    adm.global.this <- getAdmGlobal.2D(betaT = localBeta.good, G.seq = G.seq, minVal = 0, ncores = perSampleCores)  
  }
  return(adm.global.this)
}

## for each sample
samples <- unique(betaTable.ext$sample)

#id <- 1
#id <-which(samples=="TCGA-EJ-5531")

globalAdm.list <- mclapply(seq(1,length(samples),1), 
                           FUN = computeAdmGlobal2D, 
                           samples = samples, 
                           betaTable.ext = betaTable.ext, 
                           min_nsnps = min_nsnps, 
                           min_cov = min_cov, 
                           perSampleCores = perSampleCores,
                           mc.preschedule = F,
                           mc.cores = NsamplesToProcessInParallel)

globalAdm <- fromListToDF(globalAdm.list)

cat(paste("[",Sys.time() ,"] Save globald DNA admixture table in ",admTable_file,"\n",sep=""))
write.table(x=globalAdm,admTable_file,quote=F,sep="\t",col.names=T,row.names=F)

betaTable <- betaTableOriginal
rm(betaTableOriginal)