
source(paste0(path_to_CLONET_functions,"Functions/CLONET.plotFunctions.R"))

minSNPs <- min_nsnps
minCov <- min_cov

maxCN <- 10
nArms<-1000
SDcn <- 0.02


#CLONETcol <- rgb(39,144,176,maxColorValue=255)
#dataCol <- rgb(78,77,74,maxColorValue=255)

CLONETcol <- rgb(206,72,55,maxColorValue=255,alpha = 100)
NACol <- rgb(98,97,94,maxColorValue=255,alpha = 150)
clonalCol <- rgb(127,148,58, maxColorValue = 255,alpha = 150)
subclonalCol <- rgb(99,153,163, maxColorValue = 255,alpha = 150)
uncertainCol <- rgb(185,141,76, maxColorValue = 255,alpha = 150)

Xlims <- c(-1.5,1.5)
kernel.adjust <- 0.5
breaksSize <- 0.01

## Load data ----

## Clean data ----
clonalityTable.Clean <- clonalityTable[which(clonalityTable$nsnp >= min_nsnps & clonalityTable$cov >= min_cov & 
                                               (clonalityTable$chr %in% seq(1,22,1) | clonalityTable$chr %in% paste0("chr",seq(1,22,1 )) )),]


## Plot
clonalityTable.Clean.bySample <- by(clonalityTable.Clean,INDICES=clonalityTable.Clean$sample,FUN=identity)

rownames(globalAdm) <- globalAdm$sample
rownames(ploidyTable) <- ploidyTable$sample

pdf(file=reportFile,width=11,height=8.5)
#sampleID <- 23
for(sampleID in seq(1,length(clonalityTable.Clean.bySample),1)){
 
  # Load data
  thisClonality <- clonalityTable.Clean.bySample[[sampleID]]  
  
  ## assign color based on clonality Status
  thisClonality$color <- NA
  thisClonality$color[which(thisClonality$clonality.status == "clonal")] <- clonalCol
  thisClonality$color[which(thisClonality$clonality.status == "subclonal")] <- subclonalCol
  thisClonality$color[which(thisClonality$clonality.status %in% c("uncertain.clonal","uncertain.subclonal"))] <- uncertainCol
  thisClonality$color[which(thisClonality$clonality.status == "not.analysed")] <- NACol
  
  #unique(thisClonality$color)
  sampleName <- thisClonality$sample[1]
  
  #unique(clonalityTable$clonality.status)
  
  adm.local.CLONET <- round(globalAdm[sampleName,"adm"],3)
  ploidy.CLONET <- round(ploidyTable[sampleName,"ploidy"],3)
  
  minLog2 <- min(thisClonality$log2,na.rm = T)
  maxLog2 <- max(thisClonality$log2,na.rm = T)
  Breaks <- seq(floor(minLog2),ceiling(maxLog2), by = breaksSize)  
  
  
  
  
  
  layout(matrix(c(1,2,2), 3, 1, byrow = TRUE))
  
  # plot log R hist
  hist(thisClonality$log2,xlim=Xlims,breaks=Breaks, freq = F, col="gray60",border="gray60", axes=T, xlab="",main="")

  lines(density(thisClonality$log2[which(!is.na(thisClonality$log2))],adjust = kernel.adjust) , lty=1, col="tomato3", lwd = 2)
  
  abline(v=seq(0,Xlims[2],0.01),lty=3,col="gray70")
  abline(v=seq(Xlims[1],0,0.01),lty=3,col="gray70")
  
  abline(v=seq(0,Xlims[2],0.1),lty=2)
  abline(v=seq(Xlims[1],0,0.1),lty=2)
  
  abline(v=log2(seq(1,4,1)/2),lty=1)
  
  

  axis(1)
  axis(2)
  
  titleText <- paste0(sampleName,"\n",                      
                      "CLONET: purity=",1-adm.local.CLONET,"; Ploidy=",ploidy.CLONET
  )
  
  title(main=titleText)
  

  set.seed(7653)
  randomData <- data.frame(row.names=seq(1,nArms,1))
  ## fix values for ploidy
  randomData$amd.local <- rep(adm.local.CLONET,nArms)
  randomData$ploidy <- rep(ploidy.CLONET,nArms)
  possibleCNs <- getPossibleCnComb(maxCN)
  randomCNslines <- sample(seq(1,nrow(possibleCNs),1),size=nArms,replace=T)
  randomData$cnA <- possibleCNs[randomCNslines,1]
  randomData$cnB <- possibleCNs[randomCNslines,2]
  noiseCNA <- rnorm(nArms,mean=0,sd=SDcn  )
  noiseCNB <- rnorm(nArms,mean=0,sd=SDcn  )
  randomData$cnA <- pmax(0,randomData$cnA + noiseCNA)
  randomData$cnB <- pmax(0,randomData$cnB + noiseCNB)
  syntheticSegs.list <- lapply(seq(1,nrow(randomData),1),perturbDataDF,randomData)
  syntheticSegs<-cbind(randomData,do.call(rbind,syntheticSegs.list))
  plot(syntheticSegs$log2Val,syntheticSegs$betaVal.apparent,pch=16,ylim=c(0,1),xlim=c(-1.5,1.5),axes=F,xlab="",ylab="",col=CLONETcol,cex=0.7)
  
  #points(thisClonality$log2,thisClonality$beta,pch=20,col=dataCol)  

  abline(v=seq(0,Xlims[2],0.01),lty=3,col="gray70")
  abline(v=seq(Xlims[1],0,0.01),lty=3,col="gray70")
  
  abline(v=seq(0,Xlims[2],0.1),lty=2)
  abline(v=seq(Xlims[1],0,0.1),lty=2)
  
  abline(v=log2(seq(1,4,1)/2),lty=1)
  
  
  points(thisClonality$log2,thisClonality$beta,pch=20,col=thisClonality$color)  
  
  axis(1)
  axis(2)
  
  

  
  
  title(xlab="Log Ratio",ylab="Beta")  
  
  #legend(-1.5,1,legend=c("CLONET","Observed"),fill=c(CLONETcol,dataCol), bg = "white")

  legend(-1.5,1,legend=c("Expected","Clonal","Subclonal","Uncertain","Not analyzed"),
                fill=c(CLONETcol,clonalCol,subclonalCol,uncertainCol,NACol), 
         bg = "white")
  
}
dev.off()




