#### Function from Romanel et al., STM 2015.



getValsfromPDF <- function(pdf,n)
{
  return(pdf[sample(1:length(pdf),n,replace=T)])
}

BetaDistr <- function(af.control,cov.control)
{
  rep = 10000
  beta.distr = list()
  for(beta in seq(1,0.01,-0.01))
  {
    tumor = getValsfromPDF(cov.control,rep)
    tumor[which(tumor<20)] = 20
    tumor.adm = round(tumor*beta) 
    tumor.del = round(tumor-tumor.adm) 
    tumor.af = getValsfromPDF(af.control,rep)
    tumor.alt = round(tumor.adm*tumor.af)
    tumor.ref = tumor.adm-tumor.alt
    
    number = sample(1:length(tumor.ref),1)
    dir = sample(1:length(tumor.ref),number)
    
    if(beta<1)
    {
      tumor.ref[dir] = tumor.ref[dir] + tumor.del[dir]
      if(length(dir)<length(tumor.ref))
        tumor.alt[-dir] = tumor.alt[-dir] + tumor.del[-dir]
    }
    tumor.af = tumor.alt/(tumor.alt+tumor.ref)
    
    af.mirror=tumor.alt/(tumor.ref+tumor.alt)
    af.mirror[which(af.mirror<0.5)] = 1-af.mirror[which(af.mirror<0.5)]
    
    beta.distr[[length(beta.distr)+1]] = af.mirror
  }
  return(beta.distr)
}

BetaDistrMonotone <- function(beta.distr)
{
  for(i in 2:length(beta.distr))
  {
    if(median(beta.distr[[i]])<median(beta.distr[[i-1]]))
    {
      shift  = median(beta.distr[[i-1]])-median(beta.distr[[i]])
      beta.distr[[i]] = beta.distr[[i]]+shift
    }
  }
  return(beta.distr)
}

computeBeta <- function(snps.distr,beta.distr,p.thr=0.01)
{  
  betas = seq(1,0.01,-0.01)
  evidence = 1
  if (wilcox.test(snps.distr,beta.distr[[1]],alternative="greater")$p.value>p.thr)
    evidence = 0
  greater = 1
  for(i in 1:length(beta.distr))
  {
    if(wilcox.test(snps.distr,beta.distr[[i]],alternative="greater")$p.value<p.thr) { greater=i } else { break }
  }
  less = greater = i
  for(j in i:length(beta.distr))
  {
    if(wilcox.test(snps.distr,beta.distr[[j]],alternative="less")$p.value<p.thr) { less=j;break }
  }
  less = j
  if(less>greater)
    less = j-1
  q = 1-(quantile(snps.distr)-min(snps.distr))/(max(snps.distr)-min(snps.distr))
  beta = betas[less] + (betas[greater]-betas[less])*q[3]
  beta.max = betas[less] + (betas[greater]-betas[less])*q[2]
  beta.min = betas[less] + (betas[greater]-betas[less])*q[4]
  return(c(beta,beta.min,beta.max,betas[greater],betas[less],evidence))
}

#tumorBam <- samplesToAnalyse[2]
analyseSample.STMmethod<-function(tumorBam, #Name of the tumor sample to analyse
                                  samplesInfo, #File with tumor/blood sample name
                                  mutationList, #Bed list of mutations to analyse (4 column for the name)
                                  pileup_dir, #Where PaPI pileup is located
                                  analysis_dir, #Where to save the analysis of the sample
                                  minCoverage, #Min cov 
                                  min_nsnps, # min number of SNPs to call beta
                                  PaPI_Suffix, #suffix of pileup files
                                  n.digits=20, #n significative digits
                                  perSampleCores
){      
  
  cat(paste("[",Sys.time() ,"] Processing sample ",tumorBam,"\n",sep=""))
  #data structure to save information

  
  ##############
  ### Load tumor and normal pileup 
  
  #identify the sample in the samplesInfo
  sampleLine <- which(samplesInfo$Tumor.Bam.Name == tumorBam)  
  if (length(sampleLine) == 0){
    cat(paste("[",Sys.time() ,"] Error: sample ",tumorBam," is not in the samplesInfo file\n",sep=""))
    return()
  }
  
  #samplesInfo$Tumor.Bam.Name[sampleLine] <- gsub(".bam","",basename(samplesInfo$Tumor.Bam.Name[sampleLine]))
  sID.tumor <- gsub("\\.bam$","",basename(samplesInfo$Tumor.Bam.Name[sampleLine]))
  tumorPileupFileName <- paste(pileup_dir,"TumorPileup/",sID.tumor,PaPI_Suffix,sep="")  
  if (!file.exists(tumorPileupFileName)){
    cat(paste("[",Sys.time() ,"] Error: normal cannot find file  ",tumorPileupFileName,"\n",sep=""))
    return()
  }
  
  sID.normal <- gsub("\\.bam$","",basename(samplesInfo$Normal.Bam.Name[sampleLine]))
  normalPileupFileName <- paste(pileup_dir,"NormalPileup/",sID.normal,PaPI_Suffix,sep="")
  if (!file.exists(normalPileupFileName) ){ 
    cat(paste("[",Sys.time() ,"] Error: empty normal pileup file ",normalPileupFileName,"\n",sep=""))
    return()
  }
  
  tumorPileup = read.delim(tumorPileupFileName,as.is=T)
  normalPileup = read.delim(normalPileupFileName,as.is=T)
  #plot(normalPileup$af,pch=20 ,ylim=c(0,1))
  #plot(tumorPileup$af,pch=20 ,ylim=c(0,1))
  ##############
  ### select segments
  segments = mutationList[which(mutationList$sample == sID.tumor),]
  
  if (!"af" %in% colnames(normalPileup)){
    cat(paste("[",Sys.time() ,"] Error: AF for sample ",sID.normal," has to be precomputed\n",sep=""))
    return()
  }
  
  ##############
  ### filter pileup
  normalPileup <- normalPileup[which( normalPileup$af >= 0.2 & normalPileup$af <= 0.8 & normalPileup$cov >= minCoverage     ),]
  normalPileup$UID <- paste0(normalPileup$chr,":",normalPileup$pos)
  tumorPileup$UID <- paste0(tumorPileup$chr,":",tumorPileup$pos)
  tumorPileup <- tumorPileup[which( tumorPileup$cov >= minCoverage & tumorPileup$UID %in% normalPileup$UID     ),]
  
  ## rename chr X and Y if needed
  if (!bamChr){
    normalPileup$chr[which(normalPileup$chr == "X")]  <- 23
    normalPileup$chr[which(normalPileup$chr == "Y")]  <- 24
    tumorPileup$chr[which(tumorPileup$chr == "X")]  <- 23
    tumorPileup$chr[which(tumorPileup$chr == "Y")]  <- 24
    normalPileup$chr <- as.numeric(normalPileup$chr)
    tumorPileup$chr <- as.numeric(tumorPileup$chr)
  }
  
  #############  
  ### compute the distribution of beta on normal to asses noise
  beta.distr.raw = BetaDistr(normalPileup$af,normalPileup$cov)
  beta.distr = BetaDistrMonotone(beta.distr.raw)
  
  #plot(beta.distr[[1]],pch=20 ,ylim=c(0,1))
  
  
  #segID <- 24
  if (nrow(segments) > 0){
    res = mclapply(seq(1,nrow(segments),1) ,function(segID)
    {
      thisSeg <- segments[segID,,drop=F]
      
      ## init output
      thisSeg$beta = NA
      thisSeg$nref = NA
      thisSeg$nsnp = NA
      thisSeg$cov = NA
      thisSeg$sd = NA
      thisSeg$AFmean = NA
      thisSeg$AFsd = NA
      thisSeg$pval = NA
      thisSeg$delevidence = NA
      thisSeg$beta75 = NA
      thisSeg$beta25 = NA
      thisSeg$betamin = NA
      thisSeg$betamax = NA
      
      if (bamChr && !thisSeg$chr %in% paste0("chr",seq(1,22)) ){
        return(thisSeg)
      }
      
      if (!bamChr && !thisSeg$chr %in% seq(1,22) ){
        return(thisSeg)
      }
      
      
      representativeSNPs <-   tumorPileup[which((
        tumorPileup$chr == paste("chr",toString(thisSeg$chr),sep="")  |
          tumorPileup$chr == toString(thisSeg$chr)) &
          tumorPileup$pos >= as.numeric(thisSeg$start) &
          tumorPileup$pos <= as.numeric(thisSeg$end)) ,]
      
      if(nrow(representativeSNPs)>=min_nsnps)
      {
        af.seg = representativeSNPs$af
        af.seg[which(af.seg<0.5)] = 1-af.seg[which(af.seg<0.5)]
        beta = computeBeta(snps.distr=af.seg,beta.distr=beta.distr,p.thr=0.01)
        if(length(beta)>0)
        {
          thisSeg$beta = round(beta[1],digits=n.digits)
          thisSeg$nref = NA
          thisSeg$nsnp = nrow(representativeSNPs)
          thisSeg$cov = mean(representativeSNPs$cov)
          thisSeg$sd = sd(representativeSNPs$cov)
          thisSeg$AFmean = mean(representativeSNPs$af)
          thisSeg$AFsd = sd(representativeSNPs$af)
          thisSeg$pval = 0.01
          thisSeg$delevidence = round(beta[6])
          thisSeg$beta75 = round(beta[2],digits=n.digits)
          thisSeg$beta25 = round(beta[3],digits=n.digits)
          thisSeg$betamin = round(beta[5],digits=n.digits)
          thisSeg$betamax = round(beta[4],digits=n.digits)
        }
      }else{
        thisSeg$nsnp = nrow(representativeSNPs)
        thisSeg$cov = mean(representativeSNPs$cov)
        thisSeg$sd = sd(representativeSNPs$cov)
        thisSeg$AFmean = mean(representativeSNPs$af)
        thisSeg$AFsd = sd(representativeSNPs$af)
      }
      return(thisSeg)
    },mc.cores=perSampleCores)
    
  }else{
    cat(paste("[",Sys.time() ,"] Error: no segments for sample ",sID.tumor,"\n",sep=""))
    return()
  }
  
  outTable <- fromListToDF(res)
  return(outTable)
  
}
