## required CLONET libraries

##### beta table for test ----
# betaTable <- read.delim("/home/prandi/Copy/NEPC_project/CLONETruns/Results/consensuBetaTableExt_NEPC.CRPC.TCGA.BARB_badSamplesRemoved_onlyBCDE_20141204.csv",as.is=T)

##### seg files generation and analysis ----

## from beta to seg file 
fromBetaT_to_Seg <- function(betaTable, log2R.col = "log2.plCorr"){
  segFile <- betaTable[, c("sample","chr","start","end","AFmean",log2R.col)]
  return(segFile)  
}


## sample a single chr
## each step count the number of aberrant samples
sampleChr <- function(Seg.chr, start, end, step, minLogR, maxLogR, Ncores){
  
  nPoints <- floor((end-start+1)/step)+1
  chr <- Seg.chr[1,2]  
  Points <- round(seq(start,end, length.out = nPoints))
  currentPos <- Points[134]
  
  getPositionData <- function(currentPos, chr, Seg.chr, minLogR, maxLogR){
    if (currentPos > end){
      currentPos <- end
    }      
    thisSeg <- Seg.chr[which(Seg.chr[,3] <= currentPos & Seg.chr[,4] >= currentPos  ),]  
    if (length(unique(thisSeg[,1])) != nrow(thisSeg)){
      stop("Duplicated sample")
    }    
    outDF <- data.frame(nrow=1)
    outDF$chr  <- chr
    outDF$pos <- currentPos
    outDF$n.aberr <- 0
    outDF$n.sample <- 0
    outDF$percAb <- 0
    if ( nrow(thisSeg) > 0  ){
      outDF$n.sample  <- nrow(thisSeg) 
      outDF$n.aberr  <- length(which(thisSeg[,6] >=  minLogR & thisSeg[,6] <=  maxLogR )) 
      outDF$percAb <- outDF$n.aberr / outDF$n.sample
    }
    return(outDF)
  }
  
  dataChr.list <- mclapply(Points, FUN = getPositionData, chr, Seg.chr, minLogR, maxLogR, mc.preschedule = T, mc.cores = Ncores )
  dataChr <- fromListToDF(dataChr.list)
  dataChr <- dataChr[with(dataChr, order(chr,pos)),]
  return(dataChr)
#   
#   currentPos <- start  
#   dataChr <- c()  
#   #i<-1
#   #nPoints<-4
#   #currentPos <- 1000000
#   
#   
#   
#   for(i in seq(1,nPoints,1)){
#     if (currentPos > end){
#       currentPos <- end
#     }      
#     thisSeg <- Seg.chr[which(Seg.chr[,3] <= currentPos & Seg.chr[,4] >= currentPos  ),]  
#     if (length(unique(thisSeg[,1])) != nrow(thisSeg)){
#       stop("Duplicated sample")
#     }    
#     outDF <- data.frame(nrow=1)
#     outDF$chr  <- chr
#     outDF$pos <- currentPos
#     outDF$n.aberr <- 0
#     outDF$n.sample <- 0
#     outDF$percAb <- 0
#     if ( nrow(thisSeg) > 0  ){
#       outDF$n.sample  <- nrow(thisSeg) 
#       outDF$n.aberr  <- length(which(thisSeg[,6] >=  minLogR & thisSeg[,6] <=  maxLogR )) 
#       outDF$percAb <- outDF$n.aberr / outDF$n.sample
#     }
#     dataChr <- rbind(dataChr,outDF)
#     currentPos <- currentPos + step
#   }
#   return(dataChr)
}


## seg analysis ----
## given a seg file, a step (in genomic bases) and an aberration Log2R interval
## count the number of aberrant samples at each step
#hg.length <- hg19.length
analyseSegData <- function(segData,hg.length,step,minLogR, maxLogR, Ncores){
  cnData.list <- lapply(hg.length[,1], 
         function(chr, segData, minLogR, maxLogR, Ncores){
           Seg.chr <- segData[which(segData[,2] == chr),]
           start <- 1
           end <- hg.length[chr,3]    
           outData <- sampleChr(Seg.chr=Seg.chr,start=start,end=end,step=step,minLogR=minLogR,maxLogR=maxLogR, Ncores=Ncores)
           return(outData)
          }, segData, minLogR, maxLogR, Ncores)
  cnData <- fromListToDF(cnData.list)
  return(cnData)
}

### Find p-vals for differentially aberrant regions 

#Pair <- c("NEPC","CRPC")
#Sampling <- DeletionsSampling
getPvals <- function(Pair, Sampling, Ncores = 4){
  First <- Sampling[[Pair[1]]]
  Second <- Sampling[[Pair[2]]]
  if (nrow(First) != nrow(Second)){stop("Different length of samples")}
  #i<-2900
  comparePair <- function(i,First,Second){
    #      cat(i,"\n")
    if (First$chr[i] != Second$chr[i] | First$pos[i] != Second$pos[i]){stop("Different positions")}
    df.out <- data.frame(row.names = 1)
    df.out$chr <- First$chr[i]
    df.out$pos <- First$pos[i]
    df.out$n.aberr.first <- First$n.aberr[i]
    df.out$n.aberr.second <- Second$n.aberr[i] 
    df.out$n.sample.first <- First$n.sample[i]
    df.out$n.sample.second <- Second$n.sample[i] 
    
    if (First$n.sample[i] == 0 || Second$n.sample[i] == 0 ){
      df.out$pval.prop.grth <- NA
      df.out$pval.prop.loth <- NA
      df.out$pval.prop.diff <- NA
      #               df.out$pval.bin <- NA
    }else{
      ptest.gr <- prop.test(x = c(First$n.aberr[i],Second$n.aberr[i] ), n = c(First$n.sample[i], Second$n.sample[i]),correct = F, alternative = "greater")
      ptest.lo <- prop.test(x = c(First$n.aberr[i],Second$n.aberr[i] ), n = c(First$n.sample[i], Second$n.sample[i]),correct = F, alternative = "less" )
      ptest.diff<- prop.test(x = c(First$n.aberr[i],Second$n.aberr[i] ), n = c(First$n.sample[i], Second$n.sample[i]),correct = F,alternative =  "t")
      df.out$pval.prop.grth <-  ptest.gr$p.value
      df.out$pval.prop.loth <- ptest.lo$p.value
      df.out$pval.prop.diff <- ptest.diff$p.value
      #                btest <- binom.test(x = c(First$n.aberr[i],Second$n.aberr[i] ), n = c(First$n.sample[i], Second$n.sample[i]))
      #                df.out$pval.bin <-  btest$p.value
    }
    return(df.out)
  }
  
  compare.list <- mclapply(X =  seq(1,nrow(First),1),FUN = comparePair, mc.preschedule = T, mc.cores = Ncores, First=First, Second=Second )
  #compare <- do.call(rbind,compare.list)  
  compare <- fromListToDF(compare.list)
  return(compare)
}

#### allelic imbalance anlysis ----

## compute the allelic imbalance as the integer difference in the number of
## copy number between the 2 alleles
computeAllelicImbalance <- function(betaTable, 
                                    cnTh = 0.5, #the max (min) distance from the integer cn value
                                    Ncores = 4
                                     ){
  
  #round cn values
  #line<-1
  computeAllelicImbalance.line <- function(line,betaTable,cnTh){
    
    dfOut <- betaTable[line,]
    #cat(line,"\n")
    dfOut$AllelicImbalance <- NA
    dfOut$AllelicImbalance.int <- NA
    dfOut$cnA.int <- NA
    dfOut$cnB.int <- NA
    dfOut$isCNNL <- 0
    dfOut$isLOHg <- 0
    dfOut$isLOHseg <- 0
    dfOut$isUnbalancedGain <- 0
  
    if ( !is.na(dfOut$cnA) & !is.na(dfOut$cnB) ){
      if ( dfOut$cnA < floor(dfOut$cnA) + cnTh ){
        dfOut$cnA.int <- floor(dfOut$cnA)  
      }else if (dfOut$cnA >= ceiling(dfOut$cnA) - cnTh){
        dfOut$cnA.int <- ceiling(dfOut$cnA)  
      }
      
      if ( dfOut$cnB < floor(dfOut$cnB) + cnTh ){
        dfOut$cnB.int <- floor(dfOut$cnB)  
      }else if (dfOut$cnB >= ceiling(dfOut$cnB) - cnTh){
        dfOut$cnB.int <- ceiling(dfOut$cnB)  
      }
      
      dfOut$AllelicImbalance <- dfOut$cnA - dfOut$cnB
      dfOut$AllelicImbalance.int <- dfOut$cnA.int - dfOut$cnB.int    
      if (!is.na(dfOut$cnA.int) & !is.na(dfOut$cnB.int)){
        if (dfOut$cnA.int == 2 & dfOut$cnB.int == 0){
          dfOut$isCNNL <- 1
        }
        if (dfOut$cnA.int > 2 & dfOut$cnB.int == 0){
          dfOut$isLOHg <- 1
        }
        if (dfOut$cnA.int > 1 & dfOut$cnB.int == 0){
          dfOut$isLOHseg <- 1
        }      
        if (dfOut$cnA.int > 2 &  dfOut$cnB.int > 0 & ( dfOut$cnA.int - dfOut$cnB.int > 1)){
          dfOut$isUnbalancedGain <- 1
        }      
        
      }      
    }
    return(dfOut)
  }

  betaTableOut.List <- mclapply(seq(1,nrow(betaTable),1), computeAllelicImbalance.line, betaTable, cnTh, mc.preschedule = T, mc.cores = Ncores )
  #betaTableOut.List <- lapply(seq(1,nrow(betaTable),1), computeAllelicImbalance.line, betaTable, cnTh)
  betaTableOut <- fromListToDF(betaTableOut.List)
  
}

  

#### Gene level analysis ----

## given a beta table and a gene list return the segments in the beta table that intersect the genes
getGeneBasedBetaTable <- function(betaTable,
                                  geneList, # five columns table: chr  start	end	HUGO	cyto	arm
                                  Ncores = 4
                                  ){
  #geneID <- 1
  findGeneInBT <- function(geneID, betaTable, geneList){  
    betaPoses <- getSegmentsPos(geneList[geneID,"chr"], geneList[geneID,"start"], geneList[geneID,"end"], segments = betaTable[,c("chr","start","end")] )
    if (length(betaPoses) > 0){
      thisGene <- geneList[geneID,]
      if (length(colnames(geneList)) == 6){
        colnames(thisGene) <- c("gene.chr","gene.start","gene.end","HUGO","cyto","arm")  
      }else{
        colnames(thisGene)[1:4] <- c("gene.chr","gene.start","gene.end","HUGO")  
      }
      
      return(cbind(thisGene, betaTable[betaPoses,]))      
    }
    return(NULL)    
  }
  
  geneBT.list <- mclapply(seq(1,nrow(geneList),1), findGeneInBT, betaTable, geneList, mc.preschedule = T, mc.cores = Ncores)
  #geneBT.list <- mclapply(seq(1,4,1), findGeneInBT, betaTable, geneList, mc.preschedule = T, mc.cores = Ncores)
#   rbind(geneBT.list[[1]],
#   geneBT.list[[4]])
#   
  #geneBT <- do.call(rbind,geneBT.list)
  #geneBT <- rbindlist(geneBT.list)
  geneBT <- fromListToDF(geneBT.list)
  return(geneBT)    
}

## given a geneBT, i.e., a beta table with genes information for multiple samples find genes that are "differently mutated"
getDifferentlyAberrantGenes <- function(geneBT,
                                        minDist = 1, # minimum distance in the cnA vs cnB space
                                        Ncores = 4
                                        ){
  
  nSamples <- length(unique(geneBT$sample))
  Genes <- unique(geneBT$HUGO)
  gene <- Genes[1]
  testGene <- function(gene, geneBT, nSamples, minDist){
    geneBT.this <- geneBT[which(geneBT$HUGO == gene),]
    if (nrow(geneBT.this) < 2 || length(unique(geneBT.this$sample)) == 1 ){
      return(NULL)
    }
    
    toTest<-t(combn(x = nrow(geneBT.this), m = 2 ))
    distances <- sapply(seq(1,nrow(toTest),1), function(i,toTest,geneBT.this){    
      this.Dist <- sqrt(
          (geneBT.this$cnA[toTest[i,1]] - geneBT.this$cnA[toTest[i,2]])^2 +
          (geneBT.this$cnB[toTest[i,1]] - geneBT.this$cnB[toTest[i,2]])^2
        )
      return(this.Dist)
    },toTest,geneBT.this)  
    if (length(which(distances >= minDist)) > 0){
      return(gene)
    }
    return(NULL)
  }

  diffGenes.list <- mclapply(Genes,testGene, geneBT, nSamples, minDist, mc.preschedule = F, mc.cores = Ncores)
  diffGenes <- unlist(diffGenes.list) 
  return(diffGenes)
}
  
getSNVsBasedBetaTable <- function(betaTable, SNVdata, Ncores =4){
  #geneID <- 1
  if (!all(c("chr","start","end") %in% colnames(SNVdata))){
    stop("Columns chr, start, end required in SNVdata")
  }
  
  findGeneInBT <- function(geneID, betaTable, SNVdata){  
    bt <- betaTable[which(betaTable$sample == SNVdata$sample)]
    betaPoses <- getSegmentsPos(SNVdata$chr[geneID], SNVdata$start[geneID], SNVdata$end[geneID], segments = betaTable[,c("chr","start","end")] )
    if (length(betaPoses) > 0){
      thisGene <- SNVdata[geneID,]

      return(cbind(thisGene, betaTable[betaPoses,]))      
    }
    return(NULL)    
  }
  
  geneBT.list <- mclapply(seq(1,nrow(SNVdata),1), findGeneInBT, betaTable, SNVdata, mc.preschedule = T, mc.cores = Ncores)
  geneBT <- do.call(rbind,geneBT.list)
}










