# libraries
library(parallel)
library(reshape2)

## Functions for generating expected beta vs logR data ----
getPossibleCnComb <- function(maxCN,step=1){
  out<-c()
  for(i in seq(0,maxCN,step)){
    for(j in seq(0,i,step)){
      out <- rbind(out,c(j,(i-j)))
    }
  }
  return(out)
}

perturbData <- function(cnA,cnB,adm.local,MeanPloidy,coeffVariation=0.2){
  plT <- cnA+cnB 
  #pltSd <- plT * coeffVariation
  #plT <- max(0,plT + rnorm(n=1,sd=pltSd))
  plN<-2
  
  betaVal <- ( adm.local * plN )  / ( plT - adm.local * ( plT - plN ) )
  
  n.cells.mono.apparent <-  max(cnA,cnB) - min(cnA,cnB) #(cnA + cnB) - min(cnA,cnB)
  n.cells.bi.apparent <- 2*min(cnA,cnB)
  
  adm.local.apparent <- ( adm.local + (1-adm.local)*(n.cells.bi.apparent)) / ( adm.local + (1-adm.local)*(n.cells.bi.apparent) + (1-adm.local)*(n.cells.mono.apparent)  )
  betaVal.apparent <- ( adm.local.apparent * 2 )  / ( 1 + adm.local.apparent ) 
  
  #adm.local.apparent <- adm.local + (1-adm.local)*(min(cnA,cnB)/max(cnA,cnB))  
  #betaVal.apparent <- round(( adm.local.apparent * 2 )  / ( 1 + adm.local.apparent ) ,3 )
  #betaVal.apparent <- if (cnA==cnB){ 1 }else{ betaVal + (1-betaVal) * ( 2*min(cnA,cnB)/(cnA+cnB)) }  
  #adm.local.apparent <- betaVal.apparent / (2-betaVal.apparent)
  
  log2Val <- log2(( ( betaVal * plN + ( 1 - betaVal ) * plT ) / plN) / (MeanPloidy / plN))
  
  
  log2Val.corr <- log2(plT / plN)
  
  ## add noise to betaBav
  #betaValSD 
  
  #betaVal.apparent <- if (cnA==cnB){ 1 }else{ betaVal + ( 2*min(cnA,cnB)/(cnA+cnB))}  
  #((MeanPloidy/2)))  
  
  outDF <- data.frame(row.names=1)
  outDF$cnA <- cnA
  outDF$cnB <- cnB
  outDF$ploidy <- MeanPloidy
  outDF$adm.local <- adm.local
  outDF$adm.local.apparent <- adm.local.apparent
  outDF$betaVal <- betaVal
  outDF$betaVal.apparent <- betaVal.apparent
  outDF$log2Val <- log2Val
  outDF$log2Val.corr <- log2Val.corr
  outDF$log2Val.noise <- rnorm(nrow(outDF),mean=0,sd=0.05  ) + log2Val

  #outDF$log2Val.Segmentation <- log2(plT/MeanPloidy)
  return(outDF)
}

perturbDataDF <- function(i,randomData){
  return(perturbData(cnA=randomData$cnA[i],cnB=randomData$cnB[i],adm.local=randomData$amd.local[i],MeanPloidy=randomData$ploidy[i]))
}



## Functions for plotting AF data ----
plotAF <- function(refReadCount, alReadCount,Colors=c(rgb(235,159,159,maxColorValue=255),rgb(107,83,68,maxColorValue=255)),title="AF plot",...  ){
  
  barplot(names.arg=rep("",length(refReadCount)),
          rbind(refReadCount/(refReadCount+alReadCount),
                alReadCount/(refReadCount+alReadCount)),
          col=Colors,
          border=Colors)
  title(ylab="Allelic Fraction",  main=title)
  title(xlab="Informative SNPs",line=0.5)
  
}
  


## Functions for SCNA validation ----
plotSCNAValidation <- function(sampleName,geneName,RdataDir,RdataExt,globalTable,minSPNs=20,segClonality){
  
  
  SegChr = segClonality$chr[1] 
  SegStart = segClonality$start[1] 
  SegEnd = segClonality$end[1] 
  SegCN = segClonality$integerCN[1]
  SegCl = segClonality$clonality[1]
  SegClStatus = segClonality$clonality.status[1]
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  
  if(length(analysisID) == 0){
    cat("Warning",geneName,"in sample",sampleName," (Seg chr",SegChr,":",SegStart,"-",SegEnd,") is not in the RData") 
    return()
  }
  
  if(length(analysisID) > 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  ## search a clonal seg
  if ( segClonality$integerCN == 2 | segClonality$clonality.status == "not.analysed" ){
    globalTable.this <- globalTable[which(globalTable$sample == sampleName & globalTable$chr != SegChr & 
                                            globalTable$start != SegStart & globalTable$end != SegEnd & 
                                            globalTable$clonality.status == "clonal" &
                                            globalTable$integerCN == 1),]  
    if(nrow(globalTable.this) == 0){
      globalTable.this <- globalTable[which(globalTable$sample == sampleName & globalTable$chr != SegChr & 
                                              globalTable$start != SegStart & globalTable$end != SegEnd & 
                                              globalTable$integerCN == 2),]  
      
    }
    
  }else{
    globalTable.this <- globalTable[which(globalTable$sample == sampleName & globalTable$chr != SegChr & 
                                            globalTable$start != SegStart & globalTable$end != SegEnd & 
                                            globalTable$clonality.status == "clonal" &
                                            globalTable$integerCN == SegCN),]  
    
    if(nrow(globalTable.this) == 0){
      globalTable.this <- globalTable[which(globalTable$sample == sampleName & globalTable$chr != SegChr & 
                                              globalTable$start != SegStart & globalTable$end != SegEnd & 
                                              globalTable$clonality.status == "clonal" &
                                              globalTable$integerCN == 1),]  
      
    }
    
  }
  
  globalTable.this <- globalTable.this[order(globalTable.this$nsnp,decreasing=T),]
  checkChr <- globalTable.this$chr[1]
  checkStart <- globalTable.this$start[1]
  checkEnd <- globalTable.this$end[1]  
  checkCl <- globalTable.this$clonality[1]  
  checkClStatus <- globalTable.this$clonality.status[1]  
  checkID <- which(segList$chr == checkChr & segList$start == checkStart & segList$end == checkEnd)
  if(length(checkID) != 1){stop(geneName," in sample ",sampleName,": error in check segment")}
  CLONETcomputing.Validation <- dataOut$results[[checkID]]$computing
  
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(oma=c(0,0,2,0))
  par(mar=c(3,3,3,1))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=rgb(235,159,159,maxColorValue=255),altCol=rgb(107,83,68,maxColorValue=255))
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd,"\n",
                     "Clonality ",SegCl, " (",SegClStatus,")")),col.main="red4")
  
  plotAFclonet(CLONETcomputing.Validation,refCol=rgb(235,159,159,maxColorValue=255),altCol=rgb(107,83,68,maxColorValue=255))
  title(main=(paste0(checkChr,":",checkStart,"-",checkEnd,"\n",
                     "Clonality ",checkCl, " (",checkClStatus,")")),col.main="green4")
  
  par(mar=c(3,3,0,1))
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList$col[checkID] <- "green4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col,ylim=c(0,1))
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}


plotSCNAValidation.newBeta <- function(sampleName,geneName,betaTable,geneModel,tumorPileup_file,normalPileup_file){
  
  ## select data
  thisBT <- betaTable[which(betaTable$sample == sampleName),]
  geneBT <- getGeneBasedBetaTable(thisBT,geneModel[geneName,])  
  
  log2shift <- thisBT$log2shift[1]
  ## load pileup
  tumorPileup <- read.delim(tumorPileup_file,as.is=T)
  normalPileup <- read.delim(normalPileup_file,as.is=T)
  normalPileup <- normalPileup[which( normalPileup$af >= 0.2 & normalPileup$af <= 0.8 & normalPileup$cov >= 10     ),]
  
  tumorPileup.this <- tumorPileup[which(tumorPileup$chr == geneBT$chr & tumorPileup$pos >= geneBT$start & tumorPileup$pos <= geneBT$end),]
  tumorPileup.this.RefAlt <- getRefAndAltPercAbs(tumorPileup.this)
  
  normalPileup.this <- normalPileup[which(normalPileup$chr == geneBT$chr & normalPileup$pos >= geneBT$start & normalPileup$pos <= geneBT$end),]
  normalPileup.this.RefAlt <- getRefAndAltPercAbs(normalPileup.this)
  
  par(mfcol=c(2,2),oma=c(0,0,2,0))
  plot(thisBT$log2,thisBT$beta,pch=20,ylim=c(0,1),xlim=c(-1,1),xlab = "Log2R",ylab = "Beta")
  points(geneBT$log2,geneBT$beta,col="green4",pch=16)
  abline(v=log2(seq(1,4,1)/2),lty=2)
  abline(v=log2shift, lty=3, col="blue4")
  
  plot(x = thisBT$cnA,y = thisBT$cnB,pch=20,ylim=c(-0.1,2),xlim=c(-0.1,2),xlab = "cnA",ylab = "cnB",axes=F)
  points(geneBT$cnA,geneBT$cnB,col="green4",pch=16)
  axis(1,at=seq(0,2,1))
  axis(2,at=seq(0,2,1))
  abline(v=seq(0,2,1),lty=2)
  abline(h=seq(0,2,1),lty=2)  
  
  if (!is.null(geneBT) && geneBT$nsnp > 0){
    plotAF(refReadCount = tumorPileup.this.RefAlt$ref, alReadCount = tumorPileup.this.RefAlt$alt,title = "AF tumor")
    plotAF(refReadCount = normalPileup.this.RefAlt$ref, alReadCount = normalPileup.this.RefAlt$alt,title = "AF normal")
    
  }
  mtext(text = paste0(sampleName," - ",geneName),side = 3,outer = T)
  
}

## Function for PM validation
plotPMValidation <- function(sampleName,geneName,RdataDir,RdataExt,SegChr,SegStart,SegEnd,globalTable){
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  if(length(analysisID) != 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  globalTable.thisSample <- globalTable[which(globalTable$sample == sampleName),]
  globalTable.geneToCheck <- globalTable.thisSample[which(globalTable.thisSample$Gene.id == geneName),]
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(mar=c(3,3,2,1),oma=c(0,0,2,0))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=refCol,altCol=altCol)
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd)),col.main="red4")
  
  histVals <- hist(globalTable$clonality,breaks=seq(0,1,0.01),xlim=c(0,1))
  histCoord <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality) ),n=1)]
  abline(v=histCoord,col="red4")
  
  
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col,ylim=c(0,1))
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}

## Function for RR validation
plotRRValidation <- function(sampleName,geneName,RdataDir,RdataExt,SegChr,SegStart,SegEnd,globalTable){
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  if(length(analysisID) != 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  globalTable.thisSample <- globalTable[which(globalTable$sample == sampleName),]
  globalTable.geneToCheck <- globalTable.thisSample[which(globalTable.thisSample$Gene.id == geneName),]
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(mar=c(3,3,2,1),oma=c(0,0,2,0))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=refCol,altCol=altCol)
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd)),col.main="red4")
  
  histVals <- hist(globalTable$clonality,breaks=seq(0,1,0.01),xlim=c(0,1),main="")
  histCoord <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality) ),n=1)]
  histCoord.min <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality.min) ),n=1)]
  histCoord.max <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality.max) ),n=1)]
  mavVal <- max(histVals$counts)
  rect(xleft=histCoord.min,ybottom=0,xright=histCoord.max,ytop=mavVal,col=rgb(20,14,78,maxColorValue=255,alpha=140),border=rgb(20,14,78,maxColorValue=255,alpha=140) )
  abline(v=histCoord,col="red4")
  
  
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col,ylim=c(0,1))
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}


## Functions for copy number profile ----

# compute mobile average
mav <- function(x,n=5){filter(x,rep(1/n,n), sides=2)}



## plot cn profile from analyseSegData: 
## removed:now accept lists and uses plygons ----
# plotSegStatistics <- function(segStats, segColors , absolute = T , chromosomes = seq(1,22,1), ymax = NA, mavStep = 5, plotXaxis = T ){
#   
#   segStats <- segStats[which(segStats$chr %in% chromosomes),]
#   
#   if (absolute){
#     SmoothSegs <- as.vector(mav(segStats$n.aberr ,mavStep ))    
#     na.smooth <- which(is.na(SmoothSegs)) 
#     SmoothSegs[na.smooth] <- segStats$n.aberr[na.smooth]
#   }else{
#     SmoothSegs <- as.vector(mav(segStats$percAb ,mavStep ))    
#     na.smooth <- which(is.na(SmoothSegs)) 
#     SmoothSegs[na.smooth] <- segStats$percAb[na.smooth]
#   }
#   
#   ## remove NAs
#   
#   
#   
#   #which(is.na(SmoothSegs))
#   #which(is.na(segStats$percAb))
#   #SmoothSegs[which((segStats$percAb == 0))]
#   #hist(segStats$percAb,breaks=1000)
#   #barplot(SmoothSegs)
#   bpData <- barplot(SmoothSegs,plot=F)  
#   #par(mfrow=c(4,1),mar=c(0,4,0,2),oma=c(2,0,2,0))
#   
#   if (is.na(ymax)){
#     if (absolute){
#       ymax <- max( segStats$n.aberr,na.rm=T)
#       
#     }else{
#       ymax <- 1
#     }
#     
#   }
#   
#   chrStarts <- sapply(chromosomes,function(chr){min(which(segStats$chr == chr)) })  
#   chrStarts <- c(chrStarts,nrow(segStats))
#   
#   nlines <- length(bpData[chrStarts])
#   textPos <- bpData[chrStarts][1:(nlines-1)] + (bpData[chrStarts][2:nlines] - bpData[chrStarts][1:(nlines-1)])/2
#   
#   #segCol <- c("red4","green3")
#   # if more than one color is specifies, color each chr with a different color
#   if (length(segCol) > 1 & length(chrStarts) > 1){
#     cat("Warning: multiple color not yet supported with polygons. Using the first.\n")
#     SegColMulti <- segCol[1]
# #     SegColMulti <- c()
# #     nCols <- length(segCol)
# #     #segCol[3 %% ]
# #     idx<-2
# #     for(idx in seq(1,length(chrStarts)-1,1)){
# #         colIdx <- idx %% nCols
# #         if (colIdx == 0){
# #           colIdx <- nCols
# #         }
# #         thisCol <- segCol[colIdx]
# #         SegColMulti <- c(SegColMulti, rep(thisCol, chrStarts[idx+1]-chrStarts[idx] ))           
# #     }
#   }else{
#     SegColMulti <- segCol[1]
#   }
#   
#   #unique(SegColMulti)
#   
#   if (absolute){
#     #barplot(SmoothSegs,col=SegColMulti,border=SegColMulti,ylim=c(-5,ymax),axes=F)    
#     
#     plot(NA, xlim=c(0,tail(bpData,n = 1)) , ylim = c(-5,ymax),axes=F,xlab="",ylab = "")    
#     ycoords <- c(0,SmoothSegs,0) 
#     xcoords <- c(bpData[1,1],bpData[,1],tail(bpData,n = 1))    
#     polygon(x = xcoords , y = ycoords , col=SegColMulti,border = SegColMulti)
#     
#   }else{
#     
#     plot(NA, xlim=c(0,tail(bpData,n = 1)) , ylim = c(-0.3,ymax),axes=F,xlab="",ylab = "")    
#     ycoords <- c(0,SmoothSegs,0) 
#     xcoords <- c(bpData[1,1],bpData[,1],tail(bpData,n = 1))    
#     polygon(x = xcoords , y = ycoords , col=SegColMulti,border = SegColMulti)
#     
#     #barplot(SmoothSegs,col=SegColMulti,border=SegColMulti,ylim=c(-0.3,ymax),axes=F)  
#   }
#   
#   
#   
#   if (ymax == 1){
#     axisPoses <- seq(0,ymax,0.5)    
#   }else{
#     scaleStep <- max(2,floor(log10(ymax)/log10(5))*5)
#     axisPoses <- seq(0,ymax,length.out = scaleStep)
#   }
# 
#   if (absolute){
#     axis(2,at=axisPoses, labels=axisPoses, las=2)  
#   }else{
#     axisLabels <- paste0(round(axisPoses*100),"%")
#     axis(2,at=axisPoses, labels=axisLabels, las=2)
#   }
#   
#   
#   if (plotXaxis){
#     if (ymax==1){
#       segments(x0=bpData[chrStarts],y0=rep(-0.05,nlines+1),y1=rep(-0.1,nlines+1),lwd=2)
#       oddPos <- seq(1,length(textPos),2)
#       
#       text(x=textPos[oddPos],y=rep(-0.12,nlines)[oddPos],labels=chromosomes[oddPos],cex=1)
#     
#       if(length(textPos) > 1 ){
#         evenPos <- seq(2,length(textPos),2)
#         text(x=textPos[evenPos],y=rep(-0.15,nlines)[evenPos],labels=chromosomes[evenPos],cex=1)
#       }
#       
#     }else{
#       segments(x0=bpData[chrStarts],y0=rep(-0.02,nlines+1),y1=rep(-0.03,nlines+1),lwd=2)
#       oddPos <- seq(1,length(textPos),2)
#       
#       text(x=textPos[oddPos],y=rep(-0.04,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.7)
#       
#       if(length(textPos) > 1 ){
#         evenPos <- seq(2,length(textPos),2)
#         text(x=textPos[evenPos],y=rep(-0.05,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.7)
#       }
#     
#     }
# 
#     #title(xlab="Chr")
#     
#     
#   }
#   return(bpData)
#   
# }

#as.vector(mav(segStats[,6] ,mavStep ))[1:3] 

## use plolygons and accept a lsit ----
plotSegStatistics.manyData <- function(segStats.list, 
                                       neg.segStats.list = NULL, 
                                       segColors, 
                                       segColors.neg = NULL,
                                       chromosomes = seq(1,22,1), ymax = NA, mavStep = 5, plotXaxis = T ){

  #segStats <- segStats.list[[1]]
  SmoothSegs.list <- lapply(segStats.list, 
    function(segStats){
      SmoothSegs <- as.vector(mav(segStats[,6] ,mavStep ))    
      na.smooth <- which(is.na(SmoothSegs)) 
      SmoothSegs[na.smooth] <- segStats[na.smooth,6]
      return(SmoothSegs)
    })

  if (!is.null(neg.segStats.list)){
    negSmoothSegs.list <- lapply(neg.segStats.list, 
                                 function(segStats){
                                   SmoothSegs <- as.vector(mav(segStats[,6] ,mavStep ))    
                                   na.smooth <- which(is.na(SmoothSegs)) 
                                   SmoothSegs[na.smooth] <- segStats[na.smooth,6]
                                   return(SmoothSegs)
                                 })  
  }
  
  
  #length(segStats.list)
  #SmoothSegs.list[[1]]
  
  nSampling <- nrow(segStats.list[[1]])
  bpData <- barplot(SmoothSegs.list[[1]],plot=F)
    
  if (is.na(ymax)){
    ymax <- 1  
  }
  
 
  yMin <- -0.1
  chrStarts <- sapply(chromosomes,function(chr){min(which(segStats.list[[1]]$chr == chr)) })  
  chrStarts <- c(chrStarts,nSampling)
  
  nlines <- length(bpData[chrStarts])
  textPos <- bpData[chrStarts][1:(nlines-1)] + (bpData[chrStarts][2:nlines] - bpData[chrStarts][1:(nlines-1)])/2
    
  if (is.null(neg.segStats.list)){
    plot(NA, xlim=c(0,tail(bpData,n = 1)) , ylim = c(yMin,ymax),axes=F,xlab="",ylab = "")    
  }else{
    plot(NA, xlim=c(0,tail(bpData,n = 1)) , ylim = c(-ymax+yMin,ymax),axes=F,xlab="",ylab = "")    
  }
  
  xcoords <- c(bpData[1,1],bpData[,1],tail(bpData,n = 1))
#  i<-1
  for(i in seq(1,length(SmoothSegs.list),1)){
    polygon(x = xcoords , y = c(0,SmoothSegs.list[[i]],0) , col= segColors[i],border = segColors[i])  
    if (!is.null(neg.segStats.list)){
      if (is.null(segColors.neg)){
        polygon(x = xcoords , y = c(0,-negSmoothSegs.list[[i]],0) , col= segColors[i],border = NA)#segColors[i])    
      }else{
        polygon(x = xcoords , y = c(0,-negSmoothSegs.list[[i]],0) , col= segColors.neg[i],border = NA)#segColors.neg[i])    
      }
      
    }
    
    #readline()
  }
  
  
  
  if (ymax == 1){
    axisPoses <- seq(0,ymax,0.5)    
  }else{
    scaleStep <- max(2,floor(log10(ymax)/log10(5))*5)
    axisPoses <- seq(0,ymax,length.out = scaleStep)
  }
  
  axisLabels <- paste0(round(abs(axisPoses)*100),"%")
  axis(2,at=axisPoses, labels=axisLabels, las=1, cex.axis=0.8)
  if (!is.null(neg.segStats.list)){
    axis(2,at=-axisPoses, labels=axisLabels, las=1, cex.axis=0.8)  
  }
  
  yMin <- 0
  if (!is.null(neg.segStats.list)){
    yMin <- -ymax+yMin
  }
  if (plotXaxis){
    if (ymax==1){
      segments(x0=bpData[chrStarts],y0=rep(yMin,nlines+1),y1=rep(yMin-0.05,nlines+1),lwd=1,col=baseCol)
      oddPos <- seq(1,length(textPos),2)
      
      text(x=textPos[oddPos],y=rep(yMin-0.07,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.8)
      
      if(length(textPos) > 1 ){
        evenPos <- seq(2,length(textPos),2)
        text(x=textPos[evenPos],y=rep(yMin-0.1,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.8)
      }
      
    }else{
      segments(x0=bpData[chrStarts],y0=rep(-0.01,nlines+1),y1=rep(-0.02,nlines+1),lwd=1,col=baseCol)
      oddPos <- seq(1,length(textPos),2)
      
      text(x=textPos[oddPos],y=rep(-0.025,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.7)
      
      if(length(textPos) > 1 ){
        evenPos <- seq(2,length(textPos),2)
        text(x=textPos[evenPos],y=rep(-0.04,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.7)
      }
      
    }
  }
    
    #title(xlab="Chr")
    
  
  segments(x0=bpData[chrStarts],y0=rep(yMin,nlines+1),y1=ymax,lwd=1,col="white",lty=2)
  
  
  return(list(bpData=bpData, chrStarts=chrStarts, SmoothSegs.list=SmoothSegs.list))
  
  
  
}

## vrtical plot cn number statistics from analyseSegData -----
plotCNprofile <- function(segStats.del1, # deletions profile of sample1
                          segStats.del2, # deletions profile of sample2
                          segStats.amp1, # amplificatios profile of sample1
                          segStats.amp2, # amplificatios profile of sample2
                          segCol1 = "RoyalBlue", 
                          segCol2 = "Orange", 
                          absolute = T, 
                          chromosomes = seq(1,22,1), 
                          Xmax = NA, 
                          mavStep = 5, 
                          plotYaxis = T ){
  
  del1 <- segStats.del1[which(segStats.del1$chr %in% chromosomes),]
  del2 <- segStats.del2[which(segStats.del2$chr %in% chromosomes),]
  amp1 <- segStats.amp1[which(segStats.amp1$chr %in% chromosomes),]
  amp2 <- segStats.amp2[which(segStats.amp2$chr %in% chromosomes),]
  
  nSampling <- nrow(del1)
  if (!(nrow(del2) == nSampling & nrow(amp1) == nSampling & nrow(amp2) == nSampling )  ){
    stop("Sampling not uniform!")
  }

  samplingList <- list(del1=del1,del2=del2,amp1=amp1,amp2=amp2)
  
#  samplingList[["del1"]]
  if (absolute){
    samplingColumn <- "n.aberr"
  }else{
    samplingColumn <- "percAb"
  }
  
  SmoothSegs.list <- lapply(samplingList, function(segStats){
    SmoothSegs <- as.vector(mav(segStats[,samplingColumn] ,mavStep ))    
    na.smooth <- which(is.na(SmoothSegs)) 
    SmoothSegs[na.smooth] <- segStats[na.smooth,samplingColumn]
    return(SmoothSegs)
  })

  bpData <- barplot(SmoothSegs.list[["del1"]],plot=F)
  bpData[,1]  <- rev(bpData[,1])
#par(mfrow=c(4,1),mar=c(0,4,0,2),oma=c(2,0,2,0))
  
  if (is.na(Xmax)){
    if (absolute){
      perListMax <- sapply( samplingList, function(segStats){ return(max( segStats$n.aberr,na.rm=T))} )
      Xmax <- max( perListMax,na.rm=T)
    }else{
      Xmax <- 1
    }
    
  }
  
  chrStarts <- sapply(chromosomes,function(chr){min(which(samplingList[["del1"]]$chr == chr)) })  
  chrStarts <- c(chrStarts,nSampling)
  
  nlines <- length(bpData[chrStarts])
  textPos <- bpData[chrStarts][1:(nlines-1)] + (bpData[chrStarts][2:nlines] - bpData[chrStarts][1:(nlines-1)])/2
  
  #segCol <- c("red4","green3")
  # if more than one color is specifies, color each chr with a different color
#   if (length(segCol1) > 1 & length(chrStarts) > 1){
#     cat("Warning: multiple color not yet supported with polygons. Using the first.\n")
#     SegColMulti <- segCol[1]
#   }else{
#     SegColMulti <- segCol[1]
#   }
  
  #unique(SegColMulti)
  
#   if (absolute){
#     #barplot(SmoothSegs,col=SegColMulti,border=SegColMulti,ylim=c(-5,ymax),axes=F)    
#     
#     plot(NA, ylim=c(0,tail(bpData,n = 1)) , xlim = c(-Xmax,Xmax),axes=F,xlab="",ylab = "")    
#     #xcoords <- c(0,SmoothSegs.list[["del1"]],0) 
#     ycoords <- rev(c(bpData[1,1],bpData[,1],tail(bpData,n = 1)))
#     
#     polygon(x = c(0,-1*SmoothSegs.list[["del1"]],0) , y = ycoords , col=segCol1,border = segCol1)
#     polygon(x = c(0,-1*SmoothSegs.list[["del2"]],0) , y = ycoords , col=segCol2,border = segCol2)
#   
#     polygon(x = c(0,SmoothSegs.list[["amp1"]],0) , y = ycoords , col=segCol1,border = segCol1)
#     polygon(x = c(0,SmoothSegs.list[["amp2"]],0) , y = ycoords , col=segCol2,border = segCol2)
#     
#    }else{
    
     plot(NA, ylim=c(0,head(bpData,n = 1)) , xlim = c(-Xmax,Xmax),axes=F,xlab="",ylab = "")    
     #xcoords <- c(0,SmoothSegs.list[["del1"]],0) 
     ycoords <- c(bpData[1,1],bpData[,1],tail(bpData,n = 1))
     
     polygon(x = c(0,-1*SmoothSegs.list[["del1"]],0) , y = ycoords , col=segCol1,border = segCol1)
     polygon(x = c(0,-1*SmoothSegs.list[["del2"]],0) , y = ycoords , col=segCol2,border = segCol2)
     
     polygon(x = c(0,SmoothSegs.list[["amp1"]],0) , y = ycoords , col=segCol1,border = segCol1)
     polygon(x = c(0,SmoothSegs.list[["amp2"]],0) , y = ycoords , col=segCol2,border = segCol2)
    
    #barplot(SmoothSegs,col=SegColMulti,border=SegColMulti,ylim=c(-0.3,ymax),axes=F)  
  #}
  
  
  
  if (Xmax == 1){
    axisPoses <- seq(-Xmax,Xmax,0.5)    
  }else{
    scaleStep <- max(2,floor(log10(Xmax)/log10(5))*5)
    axisPoses <- seq(-Xmax,Xmax,length.out = scaleStep)
  }
  
  if (absolute){
    axis(1,at=axisPoses, labels=axisPoses, las=1)  
  }else{
    axisLabels <- paste0(round(abs(axisPoses)*100),"%")
    axis(1,at=axisPoses, labels=axisLabels, las=1, cex.axis=0.8)
  }
  
  
  if (plotYaxis){
    if (Xmax==1){
      segments(y0=bpData[chrStarts],x0=rep(-1,nlines+1),x1=rep(1,nlines+1),y1=bpData[chrStarts], lwd=1, col="white",lty=3)
      oddPos <- seq(1,length(textPos),2)
      
      text(y=textPos[oddPos],x=rep(-1,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.8,col="gray70")
      
      if(length(textPos) > 1 ){
        evenPos <- seq(2,length(textPos),2)
        text(y=textPos[evenPos],x=rep(-0.98,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.8,col="gray70")
      }
      
    }else{
      segments(x0=bpData[chrStarts],y0=rep(-0.02,nlines+1),y1=rep(-0.03,nlines+1),lwd=2)
      oddPos <- seq(1,length(textPos),2)
      
      text(x=textPos[oddPos],y=rep(-0.04,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.7)
      
      if(length(textPos) > 1 ){
        evenPos <- seq(2,length(textPos),2)
        text(x=textPos[evenPos],y=rep(-0.05,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.7)
      }
      
    }
    
    #title(xlab="Chr")
    
    
  }

  segments(x0 = 0,y0 = 0, x1 = 0,y1 = bpData[1],lwd=2,col="white")
  
  return(list(bpData=bpData, chrStarts=chrStarts, SmoothSegs.list=SmoothSegs.list))
  
}





## sample a specific region of the clonality table
sampleRegion <- function(clonalityTable, chr, start, end, nsamples, ncores=2){
  positions <- round(seq(start,end,length.out=nsamples))
  
  #pos<- positions[3]
  samplePosition <- function(pos,clonalityTable, chr){
    outDF <- data.frame(row.names=pos)
    outDF$chr <- chr
    outDF$pos <- pos
    outDF$clonal <-   length(which(clonalityTable$clonality.status == "clonal" & clonalityTable$chr == chr & clonalityTable$start <= pos & clonalityTable$end > pos))
    outDF$uncertain.clonal <- length(which(clonalityTable$clonality.status == "uncertain.clonal" & clonalityTable$chr == chr & clonalityTable$start <= pos & clonalityTable$end > pos))
    outDF$unceratin.subclonal <- length(which(clonalityTable$clonality.status == "uncertain.subclonal" & clonalityTable$chr == chr & clonalityTable$start <= pos & clonalityTable$end > pos))
    outDF$subclonal <- length(which(clonalityTable$clonality.status == "subclonal" & clonalityTable$chr == chr & clonalityTable$start <= pos & clonalityTable$end > pos))
    return(outDF)
  }
  
  Sampling.list <- mclapply(positions,samplePosition,clonalityTable=clonalityTable, chr=chr,mc.preschedule=T,mc.cores=ncores)
  #Sampling <- fromListToDF(Sampling.list)
  #Sampling <- do.call(rbind,Sampling.list)
  
  return(Sampling.list)
}


# analyse clonaity table to get number and percefextntages of clonality
analyseClonality <- function(clonalityTable,hg.length,step=step,cnValid, ncores=4){
  
  clonalityTable <- clonalityTable[which(clonalityTable$integerCN %in% cnValid),]
  
  clonalityData <- c()
  #chr<-1
  for(chr in hg.length$V1){
    Seg.chr <- clonalityTable[which(clonalityTable[,2] == chr),]
    start <- 1
    end <- hg.length[chr,3]
    
    nsamples <- floor((end-start+1)/step)+1 
    
  
    clonalityData <- rbind(clonalityData,do.call(rbind,sampleRegion(clonalityTable=clonalityTable,chr=chr,start=start,end=end,nsamples=nsamples,ncores=ncores)))    
    
  }

  return(clonalityData)
  
}


# plot clonality statistics
plotClonalityStatistics <- function(clonalityStats, chromosomes = seq(1,22,1) , mavStep = 5, plotXaxis = T){
  
  clonalityStats <- clonalityStats[which(clonalityStats$chr %in% chromosomes),]
  
  clSmooth.clonal <- as.vector(mav((clonalityStats$clonal)/
                                        (clonalityStats$clonal+clonalityStats$uncertain.clonal+clonalityStats$subclonal+clonalityStats$unceratin.subclonal) ,mavStep))
  clSmooth.subclonal <- as.vector(mav((clonalityStats$subclonal)/
                                           (clonalityStats$clonal+clonalityStats$uncertain.clonal+clonalityStats$subclonal+clonalityStats$unceratin.subclonal) ,mavStep))
  clSmooth.uncertain <- as.vector(mav((clonalityStats$uncertain.clonal + clonalityStats$unceratin.subclonal)/
                                           (clonalityStats$clonal+clonalityStats$uncertain.clonal+clonalityStats$subclonal+clonalityStats$unceratin.subclonal) ,mavStep))
  
  bpData <- barplot(clSmooth.clonal,plot=F) 
  
  chrStarts <- sapply(chromosomes,function(chr){min(which(clonalityStats$chr == chr)) })  
  chrStarts <- c(chrStarts,nrow(clonalityStats))
  
  nlines <- length(bpData[chrStarts])
  textPos <- bpData[chrStarts][1:(nlines-1)] + (bpData[chrStarts][2:nlines] - bpData[chrStarts][1:(nlines-1)])/2
  
  
  
  ymax <- 1
  
  barplot(rep(1,length(clSmooth.clonal)),col="gray80",border="gray80",ylim=c(-0.5,1),axes=F)  
  barplot(rbind(clSmooth.clonal,clSmooth.uncertain,clSmooth.subclonal),
          col=c(colors$clonal,colors$uncertain.clonal,colors$subclonal),border=c(colors$clonal,colors$uncertain.clonal,colors$subclonal)
          ,ylim=c(-0.5,1),axes=F,add=T)  
  
  #text(x=bpData[seq(1,6000,500)],y=rep(-0.3,length(seq(1,6000,500))),labels=round(clSmooth.clonal[seq(1,6000,500)],2))
  
  if (plotXaxis){
    
    segments(x0=bpData[chrStarts],y0=rep(-0.05,nlines),y1=rep(-0.08,nlines),lwd=2)
    
    oddPos <- seq(1,length(textPos),2)
    
    text(x=textPos[oddPos],y=rep(-0.12,nlines)[oddPos],labels=chromosomes[oddPos],cex=0.7)
    
    if(length(textPos) > 1 ){
      evenPos <- seq(2,length(textPos),2)
      text(x=textPos[evenPos],y=rep(-0.16,nlines)[evenPos],labels=chromosomes[evenPos],cex=0.7)
    }
  }
  
  axis(2,at=seq(0,1,0.5),labels=seq(0,1,0.5))
 # title(xlab="Chr")
  
}

# plot the logR histogram
plotLogR.hist <- function(sampleTable,Xlim, Ylim, LogRcolumn="log2.plCorr", breaksSize = 0.02, Color = "gray70"){
  minLog2 <- min(sampleTable[,LogRcolumn],na.rm = T)
  maxLog2 <- max(sampleTable[,LogRcolumn],na.rm = T)
  Breaks <- seq(floor(minLog2),ceiling(maxLog2), by = breaksSize)  
  
  hist(sampleTable[,LogRcolumn], freq = F,xlim = Xlim, breaks = Breaks, col = Color , border = Color, main="", xlab = "Log Ratio")
  Title <- paste0("Sample ",sampleTable$sample[1])
  Subtitle <- paste0("Purity ",1-sampleTable$adm.global[1]," - LogR in [",minLog2,", ",maxLog2,"]" )
  title(main=Title, line = 2)
  title(main=Subtitle, lin = 1, cex.main=1)  
  
}

## Functions for multisample comparison ----

# for each patient compute the cn status of each analyzed gene in each sample of the patient
getPatientsData <- function(GeneTable, GeneTable.good, GenesToAnalyze, cnGood , ncores=4){
  GeneTable.byPatient <- by(data=GeneTable, INDICES=GeneTable$New.Collaborator.Participant.ID, FUN=identity)
  GeneTable.good.byPatient <- by(data=GeneTable.good, INDICES=GeneTable.good$New.Collaborator.Participant.ID, FUN=identity)
  
  AvailablePatients <- names(GeneTable.byPatient)
  
  patientsData<-mclapply(AvailablePatients, FUN=getSinglePatientData,  GeneTable.byPatient,GeneTable.good.byPatient, GenesToAnalyze, cnGood , mc.preschedule=F, mc.cores=ncores)
  
  patientsData.analyzed <-patientsData[which(unlist(lapply(patientsData,function(x){!is.null(x$PatientData)})))]
  
  outData <- lapply(patientsData.analyzed, function(x){x$PatientData})
  names(outData) <- lapply(patientsData.analyzed, function(x){x$Patient})
  
  return(outData)
}

# compute the cn status of all samples from the same patient
getSinglePatientData <- function(Patient,GeneTable.byPatient,GeneTable.good.byPatient, GenesToAnalyze, cnGood = c(0,1)  ){
  
  # select patient from GeneTable
  GeneTable.patient <- GeneTable.byPatient[[Patient]]
  
  # select genes with compatible copy number 
  #GeneTable.patient.dels <- GeneTable.patient[which(GeneTable.patient$log2 >= minLogR & GeneTable.patient$log2 <= maxLogR),]
  GeneTable.patient.dels <- GeneTable.patient[which(GeneTable.patient$integer.cn %in% cnGood),]
  
  # use only Genes to Analyze
  GeneTable.patient.dels.GenesToAnalyze <- GeneTable.patient.dels[which(GeneTable.patient.dels$Gene.id %in% GenesToAnalyze),]
  
  # subset genes to analyze to hos mutated in at least one sample
  GenesToAnalyze.available <- unique(GeneTable.patient.dels.GenesToAnalyze$Gene.id)
  
  # divide patient data for each sample
  GeneTable.patient.dels.GenesToAnalyze.bySample <- 
    by( GeneTable.patient.dels.GenesToAnalyze ,INDICES=  GeneTable.patient.dels.GenesToAnalyze$sample,FUN=identity )
  
  # save clonaliy of the same patient
  clonalityTable <- GeneTable.good.byPatient[[Patient]]
  
  if (is.null(clonalityTable)){
    clonalityTable.bySample <- NULL
  }else{
    clonalityTable.bySample <- by(clonalityTable,INDICES=clonalityTable$sample, FUN=identity)  
  }
  
  patientData <- NULL
  if (length(names(GeneTable.patient.dels.GenesToAnalyze.bySample)) > 1){
    
    #Samples <- names(clonalityTable.bySample)
    Samples <- names(GeneTable.patient.dels.GenesToAnalyze.bySample)
    patientData <- data.frame(row.names=GenesToAnalyze.available)
    
    patientData[,Samples] <- notMut
        
    #Sample <- Samples[2]
    for(Sample in Samples){
      
      sampleClonality <- clonalityTable.bySample[[Sample]]
      
      genesMutated <- GeneTable.patient.dels.GenesToAnalyze$Gene.id[which(GeneTable.patient.dels.GenesToAnalyze$sample == Sample)]
      genesClonal <- sampleClonality$Gene.id[which(sampleClonality$Gene.id %in% GenesToAnalyze & sampleClonality$clonality.status == "clonal"  )]
      genesSubclonal <- sampleClonality$Gene.id[which(sampleClonality$Gene.id %in% GenesToAnalyze & sampleClonality$clonality.status == "subclonal"  )]
      genesUncertain <- sampleClonality$Gene.id[which(sampleClonality$Gene.id %in% GenesToAnalyze & sampleClonality$clonality.status %in% c("uncertain.clonal","uncertain.subclonal") )]
      
      patientData[genesMutated,Sample] <- mutated.NA
      patientData[genesClonal,Sample] <- mutated.Cl
      patientData[genesSubclonal,Sample] <- mutated.Sub
      patientData[genesUncertain,Sample] <- mutated.Un
      
    }
    
    
  }
  
  names(patientData)
  
  patientList <- NULL
  patientList$Patient <- Patient
  patientList$PatientData <- patientData
    
    
  return(patientList)
}

#patientData <- perPatient_CNstatus[["STID7520"]]
# plot patient table
plotPatientData <- function(patientData, MacroClassColor, SIF, CelColors ){
  
  patientData.toPlot <- t(as.matrix(patientData))
  xCoords <- seq(1,nrow(patientData.toPlot),1)
  yCoords <- seq(1,ncol(patientData.toPlot),1)
  zCoords <- patientData.toPlot
  xNames <- rownames(patientData.toPlot)
  yNames <- colnames(patientData.toPlot)
  
  #order columns
  MacroClassOrder <- seq(1,3,1) 
  names(MacroClassOrder) <- names(MacroClassColor) 
  samplesOrder <- order(MacroClassOrder[SIF[xNames,"MacroClasses"]])
  patientData.toPlot <- patientData.toPlot[samplesOrder,]
  xNames <- rownames(patientData.toPlot)
  
  
  axisCol <- MacroClassColor[SIF[xNames,"MacroClasses"]]
  
  par(fig=c(0,0.7,0,1))
  par(mar=c(11,9.5,2,1))
  image(x=xCoords,y=yCoords, z=zCoords, axes=F,xlab="",ylab="",col=CelColors,frame.plot=TRUE)
  box()
  grid(nx=max(xCoords), ny=max(yCoords), col="gray50", lty="solid")    
  #axis(1, at=xCoords, labels=xNames,las =2 , tick=0, col.lab="red")
  axis(1, at=xCoords, labels=F,tick=0)
  mtext(side = 1, text = xNames, at = xCoords, col = axisCol, line = 1,las=2,font=2)      
  axis(2, at=yCoords, labels=yNames,las =2 , tick=0)
  title(main=Patient)
  
  
  par(fig=c(0.7,1,0,1),new=T)
  par(mar=c(0,0,0,0))
  plot(NA,xlim=c(0,1),ylim=c(0,1),xlab="",ylab="",axes=F)
  legend(0,1, fill=CelColors, legend=c("Not mutated","Mutated NA","Clonal","Subclonal","Uncertain" ),box.lwd=0,xjust=0,yjust=1)
  
  legend(0,0.5, fill=MacroClassColor, c("HN","SC","NEPC"),box.lwd=0,xjust=0,yjust=1)
  
}


## Functions for specific genes copy number profile ----
#GeneTable.good[i,]
sampleGenesList <- function(GenesData, clonalityTable, goodCN , ncores = 4){
  
  i<-1
  sampleGene <- function(i, GenesData, clonalityTable, goodCN){
    
    # save gene data
    dfOut <- data.frame(row.names=GenesData$HUGO[i])
    dfOut$Gene.id <- GenesData$HUGO[i]
    dfOut[,colnames(GenesData)] <- GenesData[i,]
  
    
    
    dfOut[,c("n.samples","n.clonal","n.subclonal","n.uncertain")] <- NA
    
    clonalityTable.this <- clonalityTable[which(clonalityTable$Gene.id == dfOut$Gene.id & clonalityTable$integer.cn %in% goodCN),]
    dfOut$n.samples <- nrow(clonalityTable.this) 
    dfOut$n.clonal <- length(which(clonalityTable.this$clonality.status == "clonal"))
    dfOut$n.subclonal  <- length(which(clonalityTable.this$clonality.status == "subclonal"))
    dfOut$n.uncertain <-   length(which(clonalityTable.this$clonality.status %in% c("uncertain.clonal","uncertain.subclonal")))
    dfOut$tot.samples <- length(unique(clonalityTable$sample))
    dfOut$perc.samples <- round(dfOut$n.samples / dfOut$tot.samples,3)
    dfOut$perc.clonal <- round(dfOut$n.samples / dfOut$n.samples ,3)
    dfOut$perc.subclonal <- round(dfOut$n.subclonal / dfOut$n.samples,3)
    dfOut$perc.uncertain <- round(dfOut$n.uncertain / dfOut$n.samples,3)
    
    
    
    return(dfOut)
  }
  
  GenesStat.list <- mclapply(seq(1,nrow(GenesData),1),FUN=sampleGene, GenesData= GenesData,clonalityTable = clonalityTable, goodCN = goodCN, mc.cores= ncores)
  
  GenesStat <- do.call(rbind, GenesStat.list)
  return(GenesStat)  
  
}


## Function for cnA vs cnB space management ----
# G <- 0
# LogR <- -1.2
# Beta <- 0
from_BetaLogR_to_cnAB <- function(G, LogR, Beta ){
  
  cnB = ( Beta * 2^(LogR) - G) / (1 - G)
  cnA = ( (2 - Beta) * ( Beta * 2^LogR  - G)  + 2 * G * (1-Beta) ) / (  (1-G) * Beta ) 
  
  dfOut <- data.frame(cnA=cnA,cnB=cnB,stringsAsFactors=F)
  return(dfOut)
  
}

addErrorToBetaTable <- function(BetaTable, errorTable,ncores=1){
  

  betaLine <- 1
  addErrorToBetaLine <- function(betaLine, BetaTable, errorTable ){
    betas <-  BetaTable$beta[betaLine]
    nsnps <- BetaTable$nsnp[betaLine]
    cov <- BetaTable$cov[betaLine]
    
    idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
    if ( length(idxError) == 0 ){ idxError <- 1 }    
    local.errors <- errorTable$adm.estimation.error[max(idxError)]
    
    BetaTable.line <- BetaTable[betaLine,,drop=F]
    BetaTable.line$beta.error <- local.errors
    BetaTable.line$beta.min <- max(0,betas-local.errors)
    BetaTable.line$beta.max <- min(1,betas+local.errors)
    return(BetaTable.line)
    
  }
  
  extBetaTable.list <- mclapply(seq(1,nrow(BetaTable),1), addErrorToBetaLine,  BetaTable=BetaTable, errorTable=errorTable, mc.preschedule = T, mc.cores = ncores )
  extBetaTable <- do.call(rbind,extBetaTable.list)

  return(extBetaTable)
#   plot(extBetaTable$beta,extBetaTable$log2)
#   plot(extBetaTable$beta.min,extBetaTable$log2,pch=20,col="red4")
#   points(extBetaTable$beta.max,extBetaTable$log2,pch=20,col="blue4",new=T)
  
}

## Function to extend beta table with cnA and cnB 
## It is required that column log2.plCorr is defined
extendBetaTableWithCopyNumbers <- function(BetaTable, G, ncores = 1){
  #i<-1
  extendBetaLine <- function(i, BetaTable, G ){
    #cat(i,"\n")
    BetaLine <- BetaTable[i,,drop=F]
    
    cnAB.beta <- from_BetaLogR_to_cnAB(G = G, LogR = BetaLine$log2.plCorr, Beta = BetaLine$beta)
    cnAB.beta.min <- from_BetaLogR_to_cnAB(G = G, LogR = BetaLine$log2.plCorr, Beta = BetaLine$beta.min)
    cnAB.beta.max <- from_BetaLogR_to_cnAB(G = G, LogR = BetaLine$log2.plCorr, Beta = BetaLine$beta.max)
    
    BetaLine$cnA <- cnAB.beta$cnA
    BetaLine$cnB <- cnAB.beta$cnB
    BetaLine$cnA.betaMin <- cnAB.beta.min$cnA
    BetaLine$cnB.betaMin <- cnAB.beta.min$cnB
    BetaLine$cnA.betaMax <- cnAB.beta.max$cnA
    BetaLine$cnB.betaMax <- cnAB.beta.max$cnB
    return(BetaLine)
    
  }
  
  BetaTableExt.list <- mclapply(seq(1,nrow(BetaTable),1),extendBetaLine, BetaTable=BetaTable, G=G,mc.preschedule = T,mc.cores = ncores)
  BetaTableExt <- fromListToDF(BetaTableExt.list)
  return(BetaTableExt)  
}


## Function for analyzing seg files -----

# given a seg file and a list of genes coords return the number of dels and amps observed
buildAggregatedSCNAStatus <- function(SEGdata, GENESdata, delTh=-0.15, ampTh=0.15, logRcol, n.cores =4){
  
  #geneIndex <- 269
  getGeneData <- function(geneIndex, GENESdata, SEGdata){
   # cat(geneIndex,"\n")
    segmentIndexes <- getSegmentsPos(chr=GENESdata$chr[geneIndex], 
                                     initialPos=GENESdata$start[geneIndex], 
                                     finalPos=GENESdata$end[geneIndex], 
                                     segments=SEGdata[,2:4]  )  
      
    manyIntersections <- F
    if ( length(unique(SEGdata$sample[segmentIndexes]))  < length(segmentIndexes) ){
      manyIntersections <- T
    }
    
    df.out <- data.frame(row.names=GENESdata$HUGO[geneIndex])
    df.out[, colnames(GENESdata)] <- GENESdata[geneIndex,]
    df.out$n.segs <- length(segmentIndexes)
    df.out$n.samples <- length(unique(SEGdata$sample))
    df.out$manyIntersections <- manyIntersections
    df.out$delTh <- delTh
    df.out$ampTh <- ampTh
    
    #neutralSegs <- SEGdata[segmentIndexes[which(SEGdata[segmentIndexes,logRcol] >= delTh & SEGdata[segmentIndexes,logRcol] <= ampTh )],]
    #unique(neutralSegs$sample)
    #df.out$n.neutral <- 0
    #df.out$n.del <- 0
    #df.out$n.amp <- 0
        
    #sampleName <- "PM1_T_Case"
    #sampleName <- unique(thisSeg$sample)[1]
    #unique(SEGdata$sample)
    #unique(thisSeg$sample)
    thisSeg <- SEGdata[segmentIndexes,]
    CNstatus <- sapply(unique(thisSeg$sample), 
       function(sampleName, logRcol){
         #cat(sampleName,"\n")
         sampleSeg <- thisSeg[which(thisSeg$sample == sampleName),]
         if (nrow(sampleSeg) == 0){
           return("NA")
         }
         Log2R <- sampleSeg[,logRcol]
         if (nrow(sampleSeg) > 1 & !all(is.na(Log2R)) ){
           Log2R <- sampleSeg[which.max(abs(Log2R)),logRcol]
         }
         if (all(is.na(Log2R))){
           return("NA")
         }
     
        
         if ( Log2R >= delTh && Log2R <= ampTh){
          # df.out$n.neutral <- df.out$n.neutral + 1
           return("N")
         }
         if ( Log2R < delTh){
           #df.out$n.del <- df.out$n.del + 1
           return("D")
         }
         if ( Log2R > ampTh){
           #df.out$n.amp <- df.out$n.amp + 1
           return("A")
         }
       }, logRcol = logRcol)
    #length(CNstatus)
    df.out$n.neutral <- length(which(CNstatus == "N"))
    df.out$n.del <- length(which(CNstatus == "D"))
    df.out$n.amp <- length(which(CNstatus == "A"))
    df.out$n.NA <- length(which(CNstatus == "NA"))
    
    #df.out$n.neutral <- length(which(SEGdata[segmentIndexes,logRcol] >= delTh & SEGdata[segmentIndexes,logRcol] <= ampTh ))
    #df.out$n.del <- length(which(SEGdata[segmentIndexes,logRcol] < delTh))
    #df.out$n.amp <- length(which(SEGdata[segmentIndexes,logRcol] > ampTh ))
    
    return(df.out)
    
  }
  
  PerGeneSCNA.list <- mclapply(seq(1,nrow(GENESdata),1), 
                               FUN=getGeneData,  
                               GENESdata = GENESdata, 
                               SEGdata = SEGdata, 
                               mc.preschedule=T, mc.cores= n.cores )
  #getGeneData(269, GENESdata, SEGdata)
  
#   lapply(seq(1,nrow(GENESdata),1), 
#          FUN=getGeneData,  
#          GENESdata = GENESdata, 
#          SEGdata = SEGdata)
#   
  PerGeneSCNA <- do.call(rbind,PerGeneSCNA.list)
  return(PerGeneSCNA)  
  
}

# get the copy number of a gene
# CN code: 1 cn neutral, 2 deleted, 3 aplified
#getGeneCN <- function()

# given a seg file and a list of genes build table with cn status of each gene in each sample
# CN code: 1 cn neutral, 2 deleted, 3 aplified
# SEGdata <- SEGextended
# GENESdata <- goodGenes.coords
buildSCNAStatus <- function(SEGdata, GENESdata, delTh=-0.15, ampTh=0.15, logRcol,n.cores =4){
  
  samplesList <- unique(SEGdata$sample)
  nSamples <- length(samplesList)
  genesList <- unique(GENESdata$HUGO)
  nGenes <- length(genesList)
  outM <- matrix(data=NA, nrow=nSamples, ncol=nGenes)
  rownames(outM) <- samplesList
  colnames(outM) <- genesList  
  
  sample <- samplesList[4]
  gene <- genesList[12]
  for( sample in samplesList){
    thisSampleSEG <- SEGdata[which(SEGdata$sample == sample),]
    for( gene in genesList){
      thisGene <- GENESdata[gene,]
      
      segIndex <- getSegmentsPos(chr=thisGene$chr, initialPos=thisGene$start, finalPos=thisGene$end, segments=thisSampleSEG[,2:4])    
      
      if (length(segIndex) == 1){
        cnStatuses <- sapply(segIndex, FUN=
                               function(id){ 
                                 logR <- thisSampleSEG[id,logRcol] 
                                 if (is.na(logR)){ return(NA)}
                                 if (logR < delTh){ 2 }else if (logR > ampTh){ 3 }else{ 1}
                               })
        
        outM[sample,gene] <- max(cnStatuses)      
      }else{
        outM[sample,gene] <-NA
      }
      
    }
  }
  
  return(outM)
  
  #library(snow)
  #library(nws)
  
  #cl <- makeCluster(2, type = "SOCK")
  #parApply(cl=cl, X=outM, MARGIN=c(1,2), FUN=function(x)(x+1) )
  #stopCluster(cl)
  
}

# for each gene it returns logR value
buildSCNALogR <- function(SEGdata, GENESdata,logRcol,  n.cores =4, Aggr.meth="min"){
  
  samplesList <- unique(SEGdata$sample)
  nSamples <- length(samplesList)
  genesList <- unique(GENESdata$HUGO)
  nGenes <- length(genesList)
  outM <- matrix(data=NA, nrow=nSamples, ncol=nGenes)
  rownames(outM) <- samplesList
  colnames(outM) <- genesList  
  
  sample <- samplesList[4]
  gene <- genesList[12]
  for( sample in samplesList){
    thisSampleSEG <- SEGdata[which(SEGdata$sample == sample),]
    for( gene in genesList){
      thisGene <- GENESdata[gene,]
      
      segIndex <- getSegmentsPos(chr=thisGene$chr, initialPos=thisGene$start, finalPos=thisGene$end, segments=thisSampleSEG[,2:4])    
      if(length(segIndex) > 0){        
        logRs <- sapply(segIndex, FUN=
                          function(id){ 
                            logR <- thisSampleSEG[id,logRcol] 
                            return(logR)
                          })
        if (Aggr.meth == "min"){
          logR.id <- which.max(abs(logRs))
          outM[sample,gene] <- logRs[logR.id]  
        }else if (Aggr.meth == "mean"){
          outM[sample,gene] <- mean(logRs,na.rm=T)  
        }else if (Aggr.meth == "ignore"){
          ## do nothing
        }
        
        
      }
      
    }
  }
  
  return(outM)

  
}


# for each gene it returns logR value - parallel version
buildSCNALogR.parallel <- function(SEGdata, GENESdata,logRcol,  n.cores =4, Aggr.meth="min"){
  
  ## set genes data row name
  rownames(GENESdata) <- GENESdata$HUGO
  
  samplesList <- unique(SEGdata$sample)
  nSamples <- length(samplesList)
  genesList <- unique(GENESdata$HUGO)
  nGenes <- length(genesList)
#  outM <- matrix(data=NA, nrow=nSamples, ncol=nGenes)
#  rownames(outM) <- samplesList
#  colnames(outM) <- genesList  
  #i<-1
  
  getLogR <- function(i,pointsToSample,SEGdata,GENESdata){
    #cat(i,"\t")
    sample <- pointsToSample[i,1]
    gene  <- pointsToSample[i,2]
    thisSampleSEG <- SEGdata[which(SEGdata$sample == sample),]
    thisGene <- GENESdata[gene,]
    
    outDF <- data.frame(row.names = 1)
    outDF$sample <- sample
    outDF$gene <- gene
    outDF$LogR <- NA
    thisSegs <- thisSampleSEG[getSegmentsPos(chr=thisGene$chr, initialPos=thisGene$start, finalPos=thisGene$end, segments=thisSampleSEG[,2:4]),,drop=F]
    
    if(nrow(thisSegs) > 0){        
      logRs <- thisSegs[,logRcol]
      if (length(unique(logRs)) == 1 & is.na(unique(logRs))){
          ## do nothing
      }else if (Aggr.meth == "min"){
          logR.id <- which.max(abs(logRs))
          #outM[sample,gene] <- logRs[logR.id]  
          outDF$LogR <- logRs[logR.id] 
      }else if (Aggr.meth == "mean"){
          #outM[sample,gene] <- mean(logRs,na.rm=T)  
        outDF$LogR <- mean(logRs,na.rm=T)  
      }else if (Aggr.meth == "ignore"){
          ## do nothing
      }else if (Aggr.meth == "median"){
      	outDF$LogR <- median(logRs,na.rm=T) 
      }else if (Aggr.meth == "w.mean"){
      	sizes <- thisSegs$end-thisSegs$start+1
	    	outDF$LogR <- weighted.mean(logRs,sizes,na.rm=T) 
	    }
    }
    return(outDF)
  }
  
  pointsToSample <- expand.grid(samplesList,genesList,stringsAsFactors = F)

  #getLogR(1,pointsToSample = pointsToSample, SEGdata = SEGdata, GENESdata = GENESdata)
  #outTable.list <- lapply(X = seq(1,100,1), FUN = getLogR, pointsToSample = pointsToSample, SEGdata = SEGdata, GENESdata = GENESdata)
  
  outTable.list <- mclapply(X = seq(1,nrow(pointsToSample),1), FUN = getLogR, pointsToSample = pointsToSample, SEGdata = SEGdata, GENESdata = GENESdata, mc.preschedule = T, mc.cores = n.cores)
  outTable <- fromListToDF(outTable.list)
  
  outM <-  acast(outTable, sample ~ gene )
  return(outM)

  
}


### Functions for validating TEP ----

plotSCNAValidation.TEP <- function(sampleName,geneName,RdataDir,RdataExt,SegChr,SegStart,SegEnd,globalTable){
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  if(length(analysisID) != 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  ## search a clonal seg
  
  globalTable.this<- globalTable[which(globalTable$sample == sampleName & globalTable$Gene.id != geneName & globalTable$clonality.status == "clonal"),]
  globalTable.this <- globalTable.this[order(globalTable.this$nSNPs,decreasing=T),]
  checkChr <- globalTable.this$chr[1]
  checkStart <- globalTable.this$seg.start[1]
  checkEnd <- globalTable.this$seg.end[1]
  checkGene <- globalTable.this$Gene.id[1]
  checkID <- which(segList$chr == checkChr & segList$start == checkStart & segList$end == checkEnd)
  if(length(checkID) != 1){stop(geneName," in sample ",sampleName,": error in check segment")}
  CLONETcomputing.Validation <- dataOut$results[[checkID]]$computing
  
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(mar=c(3,3,2,1),oma=c(0,0,2,0))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=refCol,altCol=altCol)
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd)),col.main="red4")
  
  plotAFclonet(CLONETcomputing.Validation,refCol=refCol,altCol=altCol)
  title(main=(paste0(checkGene," ",checkChr,":",checkStart,"-",checkEnd)),col.main="green4")
  
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList$col[checkID] <- "green4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col,ylim=c(0,1))
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}

plotPMValidation.TEP <- function(sampleName,geneName,RdataDir,RdataExt,SegChr,SegStart,SegEnd,globalTable){
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  if(length(analysisID) != 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  globalTable.thisSample <- globalTable[which(globalTable$sample == sampleName),]
  globalTable.geneToCheck <- globalTable.thisSample[which(globalTable.thisSample$Gene.id == geneName),]
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(mar=c(3,3,2,1),oma=c(0,0,2,0))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=refCol,altCol=altCol)
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd)),col.main="red4")
  
  histVals <- hist(globalTable$clonality,breaks=seq(0,1,0.01),xlim=c(0,1))
  histCoord <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality) ),n=1)]
  abline(v=histCoord,col="red4")
  
  
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col)
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}

plotRRValidation.TEP <- function(sampleName,geneName,RdataDir,RdataExt,SegChr,SegStart,SegEnd,globalTable){
  
  #load RData
  load(paste0(RdataDir,sampleName,RdataExt))
  
  ## list of segments    
  segList <- lapply(dataOut$results,
                    function(x){
                      outDF <- data.frame(row.names=1)
                      mut <- unlist(x$mutation)
                      outDF$chr <- mut[1]
                      outDF$start <- as.numeric(mut[2])
                      outDF$end <- as.numeric(mut[3])
                      outDF$log2 <- as.numeric(mut[4])
                      
                      #cat(unlist(mut))
                      beta <- x$computing$pso$par[1]
                      nsnps <- length(x$repSNPs)
                      if (length(beta) == 0){ beta <- NA; nsnps <- 0}
                      outDF$beta <- beta
                      outDF$nsnps <- nsnps
                      return(outDF)
                    })
  
  segList <- fromListToDF(inputList=segList)
  
  ## retrive info about seg to check
  analysisID <- which(segList$chr == SegChr & segList$start == SegStart & segList$end == SegEnd)
  if(length(analysisID) != 1){stop(geneName," in sample ",sampleName,". Segment chr",SegChr,":",SegStart,"-",SegEnd)}
  CLONETcomputing.geneToCheck <- dataOut$results[[analysisID]]$computing
  
  globalTable.thisSample <- globalTable[which(globalTable$sample == sampleName),]
  globalTable.geneToCheck <- globalTable.thisSample[which(globalTable.thisSample$Gene.id == geneName),]
  
  
  ## plotting
  layout(mat=matrix(c(1,2,3,3),2,2,byrow=T))
  par(mar=c(3,3,2,1),oma=c(0,0,2,0))
  
  plotAFclonet(CLONETcomputing.geneToCheck,refCol=refCol,altCol=altCol)
  title(main=(paste0(geneName," ",SegChr,":",SegStart,"-",SegEnd)),col.main="red4")
  
  histVals <- hist(globalTable$clonality,breaks=seq(0,1,0.01),xlim=c(0,1),main="")
  histCoord <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality) ),n=1)]
  histCoord.min <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality.min) ),n=1)]
  histCoord.max <- histVals$breaks[tail(rank(c(histVals$mids,globalTable.geneToCheck$clonality.max) ),n=1)]
  mavVal <- max(histVals$counts)
  rect(xleft=histCoord.min,ybottom=0,xright=histCoord.max,ytop=mavVal,col=rgb(20,14,78,maxColorValue=255,alpha=140),border=rgb(20,14,78,maxColorValue=255,alpha=140) )
  abline(v=histCoord,col="red4")
  
  
  segList$col = "gray70"  
  segList$col[analysisID] <- "red4"
  segList.toPlot <- segList[which(segList$nsnps >= minSNPs),]
  plot(segList.toPlot$log2,segList.toPlot$beta,pch=20,col=segList.toPlot$col)
  segList.toPlot.focus <- segList.toPlot[which(segList.toPlot$col != "gray70"),]
  points(segList.toPlot.focus$log2,segList.toPlot.focus$beta,pch=16,col=segList.toPlot.focus$col)
  mtext(sampleName,outer=T,font=2,cex=1.4)
  
}




validateTEP <- function(dGraph,name.separator="~",list_of_clonalityTables, lossTable, gainTable, pmTable){

  
  clonalityGoodGenes.loss=list_of_clonalityTables$loss
  clonalityGoodGenes.gain=list_of_clonalityTables$gain
  clonalityGoodGenes.pm=list_of_clonalityTables$pm
  
 # this.edge.id <- E(dGraph)[6]
  for(this.edge.id in E(dGraph)){
    #sorce and end v ids
    this.edge <- E(dGraph)[this.edge.id]
    thisEdgeIDs <- get.edges(dGraph,this.edge)
    
    #v names
    vert.source.name <- strsplit(V(dGraph)$name[thisEdgeIDs[1]],name.separator)[[1]][1]
    vert.source.class <- strsplit(V(dGraph)$name[thisEdgeIDs[1]],name.separator)[[1]][2]
    vert.end.name <- strsplit(V(dGraph)$name[thisEdgeIDs[2]],name.separator)[[1]][1]
    vert.end.class <- strsplit(V(dGraph)$name[thisEdgeIDs[2]],name.separator)[[1]][2]
    
    occ <- this.edge$occurrency
    dep <- this.edge$dependency
    
    #samples with the source
    sourceTable <- get(paste0("clonalityGoodGenes.",vert.source.class))
    endTable <-  get(paste0("clonalityGoodGenes.",vert.end.class))
    
    
    # count occurrency and dependency from tables
    sourceTable.gene <- sourceTable[which(sourceTable$Gene.id == vert.source.name),]
    endTable.gene <- endTable[which(endTable$Gene.id == vert.end.name),]
    sourceTable.gene.clonal <- sourceTable[which(sourceTable$Gene.id == vert.source.name &  sourceTable$clonality.status %in% clonalStatus ),]
    endTable.gene.subclonal <- endTable[which(endTable$Gene.id == vert.end.name & endTable$clonality.status %in% subclonalStatus),]
    globalTableSource <- get(paste0(vert.source.class,"Table"))
    globalTableEnd <- get(paste0(vert.end.class,"Table"))
    occurrencySamples <- intersect(sourceTable.gene$sample,endTable.gene$sample)
    depSamples <- intersect(sourceTable.gene.clonal$sample, endTable.gene.subclonal$sample )
    
    # check if table and graph reports the same dep and occ
    if ( length(occurrencySamples) != occ || length(depSamples) != dep ){
      warning(vert.source.name,"~",vert.source.class," -> ",vert.end.name,"~",vert.end.class,": occ or dep wrong!")
    }
    
    #thisSample <- depSamples[1]
    for(thisSample in depSamples){
      
      sourceSegChr <- sourceTable.gene.clonal$chr[which(sourceTable.gene.clonal$sample == thisSample)]
      sourceSegStart <- sourceTable.gene.clonal$seg.start[which(sourceTable.gene.clonal$sample == thisSample)]
      sourceSegEnd <- sourceTable.gene.clonal$seg.end[which(sourceTable.gene.clonal$sample == thisSample)]  
      if (vert.source.class %in% c("loss","gain")){      
        plotSCNAValidation.TEP(sampleName=thisSample,geneName=vert.source.name,
                           RdataDir=RdataDir,RdataExt=RdataExt,SegChr=sourceSegChr,SegStart=sourceSegStart,
                           SegEnd=sourceSegEnd,globalTable=globalTableSource)
        
      }else if (vert.source.class %in% c("pm")){ 
        plotPMValidation.TEP(sampleName=thisSample,geneName=vert.source.name,
                         RdataDir=RdataDir,RdataExt=RdataExt,SegChr=sourceSegChr,SegStart=sourceSegStart,
                         SegEnd=sourceSegEnd,globalTable=globalTableSource)
        
      }else if (vert.source.class %in% c("rr")){
        plotRRValidation.TEP(sampleName=thisSample,geneName=vert.source.name,
                         RdataDir=RdataDir,RdataExt=RdataExt,SegChr=sourceSegChr,SegStart=sourceSegStart,
                         SegEnd=sourceSegEnd,globalTable=globalTableSource)
        
      }
      
      
      endSegChr <- endTable.gene.subclonal$chr[which(endTable.gene.subclonal$sample == thisSample)]
      endSegStart <- endTable.gene.subclonal$seg.start[which(endTable.gene.subclonal$sample == thisSample)]
      endSegEnd <- endTable.gene.subclonal$seg.end[which(endTable.gene.subclonal$sample == thisSample)]
      
      if (vert.end.class %in% c("loss","gain")){
        plotSCNAValidation.TEP(sampleName=thisSample,geneName=vert.end.name,
                           RdataDir=RdataDir,RdataExt=RdataExt,SegChr=endSegChr,SegStart=endSegStart,
                           SegEnd=endSegEnd,globalTable=globalTableEnd)
      }else if (vert.end.class %in% c("pm")){ 
        plotPMValidation.TEP(sampleName=thisSample,geneName=vert.end.name,
                         RdataDir=RdataDir,RdataExt=RdataExt,SegChr=endSegChr,SegStart=endSegStart,
                         SegEnd=endSegEnd,globalTable=globalTableEnd)
        
      }
      
      
    }
  }
}



### Plot allelic imbalance circos ----
plotAllelicImbalanceCircos <- function(
  betaTableExt,
  sampleToPlot,
  minSNPs,
  yMax,
  cnBcolor = "red",
  cnAcolor ="blue",
  NAcol = "gray60" ){
  
  
  
  ## prepare data
  betaTableExt.toPlot <- betaTableExt[which(betaTableExt$sample == sampleToPlot),]  
  betaTableExt.toPlot <- betaTableExt.toPlot[with(betaTableExt.toPlot,order(chr,start)),]
  
  if ("X" %in% betaTableExt.toPlot$chr){
    
    ## laod data and libraries
    ## load ideogram
    data("hg19Ideogram", package = "biovizBase")
    chr.sub <- c(paste("chr", 1:22, sep = ""),"chrX")
    new.names <- as.character(1:23)
    names(new.names) <- chr.sub #paste("chr", new.names, sep = "")
    hg19Ideo <- keepSeqlevels(hg19Ideogram, chr.sub)
    hg19Ideo <- renameSeqlevels(hg19Ideo, new.names)
    head(hg19Ideo)
    seqlevels(hg19Ideo)
    
    ## add chrX 
    betaTableExt.toPlot$Xcn <- 2^(betaTableExt.toPlot$log2.plCorr)
    betaTableExt.toPlot$cnA.int[which(betaTableExt.toPlot$chr == 23)] <- betaTableExt.toPlot$Xcn[which(betaTableExt.toPlot$chr == 23)]
    betaTableExt.toPlot$cnB.int[which(betaTableExt.toPlot$chr == 23)] <- 0  
    
  }else{
    data("hg19Ideogram", package = "biovizBase")
    chr.sub <- paste("chr", 1:22, sep = "")
    new.names <- as.character(1:22)
    names(new.names) <- chr.sub #paste("chr", new.names, sep = "")
    hg19Ideo <- keepSeqlevels(hg19Ideogram, chr.sub)
    hg19Ideo <- renameSeqlevels(hg19Ideo, new.names)
    head(hg19Ideo)
    seqlevels(hg19Ideo)
    
  
    
  }
  
  
  
  
  
  
  ## mark low quality
  lowQual <- which(betaTableExt.toPlot$nsnp <= minSNPs)
  betaTableExt.toPlot$cnA.int[lowQual] <- 0
  betaTableExt.toPlot$cnB.int[lowQual] <- 0  
  betaTableExt.toPlot$cnA.toPlot <- betaTableExt.toPlot$cnA.int 
  betaTableExt.toPlot$cnB.toPlot <- betaTableExt.toPlot$cnB.int   
  betaTableExt.toPlot$cnA.toPlot[which(is.na(betaTableExt.toPlot$cnA.int))] <- 0
  betaTableExt.toPlot$cnB.toPlot[which(is.na(betaTableExt.toPlot$cnB.int))] <- 0
  
  ##set colors
  betaTableExt.toPlot$cnAcol <- cnAcolor
  betaTableExt.toPlot$cnBcol <- cnBcolor
  
  ## create genomic ranges object
  GR.toPlot <-  GRanges(seqnames = factor(c(1,betaTableExt.toPlot$chr), levels = seqlevels(hg19Ideo) ),
                        IRanges(
                          start = c(0,betaTableExt.toPlot$start),
                          end = c(0,betaTableExt.toPlot$end)                          
                        ),
                        cnA = pmin(c(yMax,betaTableExt.toPlot$cnA.toPlot),6),
                        cnB = pmin(c(yMax,betaTableExt.toPlot$cnB.toPlot),6),
                        cnAcol = c(NAcol,betaTableExt.toPlot$cnAcol),
                        cnBcol = c(NAcol,betaTableExt.toPlot$cnBcol),
                        dummy = 1#rep(0,1+nrow(betaTableExt.toPlot))
  )
  
  
  
  fillColors <- c(cnBcolor,NAcol,cnAcolor)
  gridBg <- rgb(red = 200,200,200,alpha = 0,maxColorValue = 255)
  
  ## plot
  p<-  ggplot() +  
    layout_circle(hg19Ideo,geom = "ideo", fill = "gray70",radius = 85, trackWidth = 1, colour= "gray70") +
    layout_circle(hg19Ideo, geom = "text", aes(label = seqnames), vjust = 0,radius = 72, trackWidth = 12, size = 5) +
    #    layout_circle(GR.toPlot, geom = "bar", radius = 27, trackWidth = 45, size=1,color="red",fill="red",aes( y = cnA ), grid=T, grid.n=yMax+1) 
    layout_circle(GR.toPlot, geom = "bar", radius = 27, trackWidth = 45, size=0.5, color=fillColors[3], fill=fillColors[3],aes( y = cnA ), 
                  grid=T, grid.n=yMax+1)  +
    layout_circle(GR.toPlot, geom = "bar", radius = 27, trackWidth = 45, size=1,color=fillColors[1], fill=fillColors[1],aes( y = cnB)) +
    layout_circle(GR.toPlot, geom ="line",radius = 27, trackWidth = 45, size=1,color = "white",aes( y = dummy),  grid=T, grid.n=yMax+1, grid.background = gridBg, grid.line = "white" ) + 
    scale_fill_manual(values = fillColors, guide=guide_legend(title=""), labels=c("Allele B","N.D.","Allele A")) +    
    scale_color_manual(values = fillColors) +
    labs(title=sampleToPlot)

  
  print(p)

}
